#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtCore, QtGui, Qt, QtWidgets, uic
from pkg_resources import resource_filename
import datetime
import tempfile
from icepap import State, Mode
from xml.dom import minidom, Node
from xml.dom.minidom import getDOMImplementation
import logging
from .Led import Led
from ..lib import MainManager, IcepapsManager, ConfigManager, \
    StormManager
from .messagedialogs import MessageDialogs
from ..helpers import loggingInfo
from .dialoghomesrch import DialogHomeSrch


class PageiPapDriver(QtWidgets.QWidget):
    """
    Widget that manages all the information related to an
    icepap driver. Configuration, testing and historic configurations
    """
    log = logging.getLogger('{}.PageiPapDriver'.format(__name__))

    @loggingInfo
    def __init__(self, mainwin, test_mode=False):
        QtWidgets.QWidget.__init__(self, None)
        self._mainwin = mainwin
        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'pageipapdriver.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui,
                   package='icepapcms.gui')

        self.signalMapper = QtCore.QSignalMapper(self)
        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'axis.ui')
        self.axis = QtWidgets.QDialog()
        uic.loadUi(ui_filename, baseinstance=self.axis,
                   package='icepapcms.gui')


        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'motor.ui')
        self.motor = QtWidgets.QDialog()
        uic.loadUi(ui_filename, baseinstance=self.motor,
                   package='icepapcms.gui')


        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'encoders.ui')
        self.encoders = QtWidgets.QDialog()
        uic.loadUi(ui_filename, baseinstance=self.encoders,
                   package='icepapcms.gui')

        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'closedloop.ui')
        self.closedloop = QtWidgets.QDialog()
        uic.loadUi(ui_filename, baseinstance=self.closedloop,
                   package='icepapcms.gui')

        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'homing.ui')
        self.homing = QtWidgets.QDialog()
        uic.loadUi(ui_filename, baseinstance=self.homing,
                   package='icepapcms.gui')

        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'io.ui')
        self.io = QtWidgets.QDialog()
        uic.loadUi(ui_filename, baseinstance=self.io,
                   package='icepapcms.gui')


        self.param_to_widgets = {}
        self.ui_widgets = []

        self.param_to_unknown_widgets = {}

        self.tab_frames = [
            self.axis,
            self.motor,
            self.encoders,
            self.closedloop,
            self.homing,
            self.io]
        self.tab_labels = ["Axis", "Motor", "Encoders", "Closed loop",
                           "Homing", "I/O"]

        for index in range(len(self.tab_labels)):
            widget = self.tab_frames[index]
            label = self.tab_labels[index]
            self.ui.tabWidget.insertTab(index, widget, label)

        for index in range(self.ui.tabWidget.count()):
            page_widget = self.ui.tabWidget.widget(index)
            for widget in page_widget.findChildren(
                    QtWidgets.QWidget, QtCore.QRegExp('^cfg|^cmd|^txt|^chk')):
                widget_name = str(widget.objectName())
                param_name = str(widget_name[3:])
                if param_name in self.param_to_widgets:
                    self.param_to_widgets.get(param_name).append(widget)
                else:
                    self.param_to_widgets[param_name] = [widget]
                widget.param = param_name
                widget.defaultvalue = None
                widget.tab_index = index
                widget.isCommand = widget_name.startswith("cmd")
                self.ui_widgets.append(widget)
                self._connectWidgetToSignalMap(widget)

        # WE CREATE THE UNKNOWN_TAB FOR NEW_PARAMETERS THE APPLICATION DOES NOT
        # KNOW
        self.unknown_tab, self.unknown_table_widget = self.createTableWidget(
            ["Name", "Value", "New Value"])
        self.lastTabSelected = 0
        if test_mode:
            return

        self.widgets_modified = []
        # THIS LIST SHOULD BE POPULATED WITH DRIVERS BUT IT IS NOT SUFFICIENT
        # BECAUSE THE __cmp__ METHOD OF THE ICEPAPDRIVER SETS CONFLICTS AND WE
        # JUST WANT TO SEARCH, INSTEAD OF THE DRIVER OBJECT IT WILL BE
        # POPULATED BY THE KEY COMPOSED BY: ICEPAP_SYSTEM + "_" + DRIVER_ADDR
        self.saveConfigPending = []
        self.tabs_modified = {}
        self.tabs_configPending = {}
        self.ui.tabWidget.setCurrentIndex(self.lastTabSelected)

        self.refreshTimer = Qt.QTimer(self)
        self.sliderTimer = Qt.QTimer(self)
        self.sliderTimer.setInterval(100)

        self.signalConnections()

        self._setWidgetToolTips()

        self._manager = MainManager()
        self.ui.btnUndo.setEnabled(False)

        self.setLedsOff()
        self.icepap_driver = None
        self.inMotion = -1
        self.status = -1

        self.ui.historicWidget.setCfgPage(self)
        self.hideHistoricWidget()

        self.ui.sliderJog.setEnabled(False)

        # PALETTES TO HIGHLIGHT WIDGETS
        white_brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        grey_brush = QtGui.QBrush(QtGui.QColor(239, 235, 231))
        yellow_brush = QtGui.QBrush(QtGui.QColor(255, 255, 0))
        salmon_brush = QtGui.QBrush(QtGui.QColor(255, 206, 162))
        blue_brush = QtGui.QBrush(QtGui.QColor(135, 206, 250))

        self.base_white_palette = QtGui.QPalette()
        self.base_yellow_palette = QtGui.QPalette()
        self.base_salmon_palette = QtGui.QPalette()

        self.button_grey_palette = QtGui.QPalette()
        self.button_yellow_palette = QtGui.QPalette()
        self.button_salmon_palette = QtGui.QPalette()
        self.button_blue_palette = QtGui.QPalette()

        self.base_white_palette.setBrush(
            QtGui.QPalette.Active, QtGui.QPalette.Base, white_brush)
        self.base_white_palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.Base,
            white_brush)
        self.base_yellow_palette.setBrush(
            QtGui.QPalette.Active, QtGui.QPalette.Base, yellow_brush)
        self.base_yellow_palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.Base,
            yellow_brush)
        self.base_salmon_palette.setBrush(
            QtGui.QPalette.Active, QtGui.QPalette.Base, salmon_brush)
        self.base_salmon_palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.Base,
            salmon_brush)

        self.button_grey_palette.setBrush(
            QtGui.QPalette.Active, QtGui.QPalette.Button, grey_brush)
        self.button_grey_palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.Button,
            grey_brush)
        self.button_yellow_palette.setBrush(
            QtGui.QPalette.Active, QtGui.QPalette.Button, yellow_brush)
        self.button_yellow_palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.Button,
            yellow_brush)
        self.button_salmon_palette.setBrush(
            QtGui.QPalette.Active, QtGui.QPalette.Button, salmon_brush)
        self.button_salmon_palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.Button,
            salmon_brush)
        self.button_blue_palette.setBrush(
            QtGui.QPalette.Active, QtGui.QPalette.Button, blue_brush)
        self.button_blue_palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.Button,
            blue_brush)

        # PALETTES TO HIGHLIGHT THE DESCRIPTION QFRAME
        light_blue_brush = QtGui.QBrush(QtGui.QColor(224, 255, 255))
        self.qframe_lightblue_palette = QtGui.QPalette()
        self.qframe_salmon_palette = QtGui.QPalette()
        self.qframe_lightblue_palette.setBrush(
            QtGui.QPalette.Active, QtGui.QPalette.Window, light_blue_brush)
        self.qframe_lightblue_palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.Window,
            light_blue_brush)
        self.qframe_salmon_palette.setBrush(
            QtGui.QPalette.Active, QtGui.QPalette.Window, salmon_brush)
        self.qframe_salmon_palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.Window,
            salmon_brush)

        self.dbStartupConfig = None

    @loggingInfo
    def signalConnections(self):
        self.ui.btnBlink.pressed.connect(self.btnBlink_on_press)
        self.ui.tabWidget.currentChanged.connect(self.tabWidget_currentChanged)
        self.ui.btnSendCfg.clicked.connect(self.btnSendCfg_on_click)
        self.ui.btnSaveCfg.clicked.connect(self.btnSaveCfg_on_click)
        self.refreshTimer.timeout.connect(self.updateTestStatus)
        self.ui.btnEnable.clicked.connect(self.endisDriver)
        self.ui.txtSpeed.editingFinished.connect(self.setMotionValues)
        self.ui.txtAcceleration.editingFinished.connect(self.setMotionValues)
        self.ui.btnGO.clicked.connect(self.btnGO_on_click)
        self.ui.btnGORelativePos.clicked.connect(
            self.btnGORelativePos_on_click)
        self.ui.btnGORelativeNeg.clicked.connect(
            self.btnGORelativeNeg_on_click)
        self.ui.btnStopMotor.clicked.connect(self.btnStopMotor_on_click)
        self.ui.btnSetPos.clicked.connect(self.setPosition)
        self.ui.btnSetEnc.clicked.connect(self.setEncoder)
        self.ui.btnUndo.clicked.connect(self.btnUndo_on_click)
        self.ui.btnRestore.clicked.connect(self.btnRestore_on_click)
        self.ui.sliderJog.sliderMoved.connect(self.startJogging)
        self.ui.sliderJog.valueChanged.connect(self.sliderChanged)
        self.ui.sliderJog.sliderReleased.connect(self.stopJogging)
        self.ui.sliderJog.valueChanged.connect(self.sliderChanged)
        self.sliderTimer.timeout.connect(self.resetSlider)
        self.ui.cmdCSWITCH.currentIndexChanged.connect(
            self.changeSwitchesSetup)
        self.ui.btnHomeSrch.clicked.connect(self._display_home_srch_dialog)

    def enable_home_srch_button(self):
        """Enables the HOME/SRCH button."""
        self.ui.btnHomeSrch.setDisabled(False)

    def _display_home_srch_dialog(self):
        system = self.icepap_driver.icepapsystem_name
        addr = self.icepap_driver.addr
        axis = IcepapsManager().iPaps[system][addr]
        if Mode.CONFIG in axis.mode:
            msg = 'Axis is in config mode. Aborting'
            print(msg)
            MessageDialogs.showErrorMessage(None, 'HOME/SRCH', msg)
        dlg = DialogHomeSrch(self, axis)
        dlg.show()
        self.ui.btnHomeSrch.setDisabled(True)

    @loggingInfo
    def highlightWidget(self, widget):
        # AGAIN, COMMAND WIDGETS ARE ONLY CHECKED IN THE QCOMBOBOX
        # ELIF SECTION
        if not isinstance(widget, QtWidgets.QWidget):
            self.log.error("THIS WIDGET SHOULD NOT BE INTENDED TO "
                           "HIGHLIGHT... NOT A QWIDGET %s",
                           widget.objectName())
            return

        if widget.defaultvalue is None:
            widget_name = widget.objectName()
            if widget_name.startswith("txt"):
                # PASS IT IS NORMAL TO NOT HIGHLIGHT IT, JUST INFO
                # txtNCURR, txtPCLOOP
                pass
            elif widget_name.count("_") > 0:
                # IT IS A FLAG OF A FLAGS PARAMETER, SO IT'S PARENT SHOULD BE
                # HIGHLIGHTED
                self.highlightWidget(widget.parent())
            else:
                pass
            return

        highlight = False
        sendConfig = False
        saveConfig = False

        param = widget.param
        if param == "DriverName":
            param = "IPAPNAME"

        dbvalue = self.dbStartupConfig.getParameter(
            str(param), in_memory=False)
        if dbvalue is None:
            dbvalue = ""
            # SPECIAL CASE IS THE FLAGS PARAMETER WHICH IS STORED
        wvalue = self._getWidgetValue(widget)
        defvalue = widget.defaultvalue
        if defvalue is None:
            defvalue = ""

        try:
            if isinstance(widget, QtWidgets.QDoubleSpinBox) or isinstance(
                    widget, QtWidgets.QSpinBox):
                if widget.defaultvalue != wvalue:
                    highlight = True
                    sendConfig = True
                    widget.setPalette(self.base_yellow_palette)
                elif abs(float(wvalue) - float(dbvalue)) > 0.01:
                    saveConfig = True
                    widget.setPalette(self.base_salmon_palette)
                else:
                    widget.setPalette(self.base_white_palette)

            elif isinstance(widget, QtWidgets.QCheckBox):
                if widget.defaultvalue != wvalue:
                    highlight = True
                    sendConfig = True
                    widget.setPalette(self.base_yellow_palette)
                elif wvalue != dbvalue:
                    saveConfig = True
                    widget.setPalette(self.base_salmon_palette)
                else:
                    widget.setPalette(self.base_white_palette)

            elif isinstance(widget, QtWidgets.QComboBox):
                try:
                    dbvalue = dbvalue
                    wvalue = wvalue
                    defvalue = defvalue
                    if widget.defaultvalue is None:
                        widget.defaultvalue = ""
                    elif defvalue != wvalue:
                        highlight = True
                        sendConfig = True
                        widget.setPalette(self.button_yellow_palette)
                        # SPECIAL CASE WITH COMMAND WIDGETS
                        if widget.isCommand:
                            widget.setPalette(self.button_blue_palette)
                    elif wvalue != dbvalue:
                        saveConfig = True
                        widget.setPalette(self.button_salmon_palette)
                        # SPECIAL CASE WITH COMMAND WIDGETS
                        if widget.isCommand:
                            saveConfig = False
                            widget.setPalette(self.button_blue_palette)

                    else:
                        widget.setPalette(self.button_grey_palette)
                except Exception as e:
                    self.log.error("some exception found trying to "
                                   "highlight a QComboBox %s!: %s",
                                   widget.objectName(), e)

            elif isinstance(widget, QtWidgets.QLineEdit):
                if dbvalue is None:
                    dbvalue = ""
                dbvalue = dbvalue
                wvalue = wvalue
                defvalue = defvalue
                if defvalue is None:
                    widget.defaultvalue = ""
                    defvalue = ""

                if defvalue != wvalue:
                    highlight = True
                    sendConfig = True
                    widget.setPalette(self.base_yellow_palette)
                elif wvalue != dbvalue:
                    saveConfig = True
                    widget.setPalette(self.base_salmon_palette)
                else:
                    widget.setPalette(self.base_white_palette)

            elif isinstance(widget, QtWidgets.QFrame):
                regexp = QtCore.QRegExp('^' + widget.objectName() + "_")
                for w in self.ui.tabWidget.findChildren(QtWidgets.QWidget,
                                                        regexp):
                    w_param_str = str(w.objectName())
                    w_param = "_".join(w_param_str.split("_")[1:])
                    defvalue_count = defvalue.count(w_param)
                    dbvalue_count = dbvalue.count(w_param)
                    if (w.isChecked() and (defvalue_count == 0)) or (
                            not w.isChecked() and defvalue_count > 0):
                        highlight = True
                        sendConfig = True
                        w.setPalette(self.base_yellow_palette)
                    elif (w.isChecked() and (dbvalue_count == 0)) or \
                            (not w.isChecked() and dbvalue_count > 0):
                        saveConfig = True
                        w.setPalette(self.base_salmon_palette)
                    else:
                        w.setPalette(self.base_white_palette)

        except Exception as e:
            self.log.error("Some exception found with param %s on method "
                           "highlightWidget: %s", param, e)

        if highlight:
            if widget not in self.widgets_modified:
                self.widgets_modified.append(widget)
        else:
            if widget in self.widgets_modified:
                self.widgets_modified.remove(widget)

        driver_key = self.icepap_driver.icepapsystem_name + \
            "_" + str(self.icepap_driver.addr)
        if saveConfig and driver_key not in self.saveConfigPending:
            self.saveConfigPending.append(driver_key)

        tab_index = widget.tab_index
        index_in_tabs_modified = tab_index in self.tabs_modified
        index_in_tabs_configPending = tab_index in self.tabs_configPending
        if highlight:
            if index_in_tabs_modified:
                self.tabs_modified.get(tab_index).append(widget)
            else:
                self.tabs_modified[tab_index] = [widget]
        elif saveConfig:
            if index_in_tabs_configPending:
                self.tabs_configPending.get(tab_index).append(widget)
            else:
                self.tabs_configPending[tab_index] = [widget]
        else:
            if index_in_tabs_modified:
                widget_list = self.tabs_modified.get(tab_index)
                if widget in widget_list:
                    widget_list.remove(widget)
                    if len(widget_list) == 0:
                        self.tabs_modified.pop(tab_index)
            if index_in_tabs_configPending:
                widget_list = self.tabs_configPending.get(tab_index)
                if widget in widget_list:
                    widget_list.remove(widget)
                    if len(widget_list) == 0:
                        self.tabs_configPending.pop(tab_index)

        # ONLY ALLOW SEND/SAVE CONFIG BUTTONS IF CONFIG NEEDED
        # Change behaviour in version 1.23
        enable_send = len(self.widgets_modified) > 0
        enable_save = driver_key in self.saveConfigPending and not sendConfig
        self.ui.btnSendCfg.setEnabled(enable_send)
        self.ui.btnSaveCfg.setEnabled(enable_save)
        self._mainwin.ui.actionSaveConfig.setEnabled(enable_save)

    @loggingInfo
    def _connectHighlighting(self):
        self.signalMapper.mapped[QtWidgets.QWidget].connect(
            self.highlightWidget)

    @loggingInfo
    def _disconnectHighlighting(self):
        try:
            self.signalMapper.mapped[QtWidgets.QWidget].disconnect(
                self.highlightWidget)
        except Exception:
            pass

    @loggingInfo
    def _setWidgetToolTips(self):
        """ Reads the driverparameters file and sets the tooltips"""

        UI_PARS = list(self.param_to_widgets.keys())
        FOUND_PARS = []
        NOT_FOUND_PARS = []
        MISSING_TOOLTIPS = []

        driverparameters = resource_filename('icepapcms.templates',
                                             'driverparameters.xml')

        doc = minidom.parse(driverparameters)
        root = doc.documentElement
        for section in root.getElementsByTagName("section"):
            for pars in section.getElementsByTagName("par"):
                if pars.nodeType == Node.ELEMENT_NODE:
                    parid = pars.attributes.get('id').value
                    parid = parid.strip()
                    parname = pars.attributes.get('name').value
                    parname = parname.strip()
                    pardesc = self._getText(
                        pars.getElementsByTagName("description")[0].firstChild)

                    # SHOULD I USE pardesc as the tooltip?
                    try:
                        if parid in self.param_to_widgets:
                            widgets = self.param_to_widgets[parid]
                            for w in widgets:
                                w.setToolTip(pardesc)
                            FOUND_PARS.append(parid)
                        else:
                            NOT_FOUND_PARS.append(parid)
                    except Exception as e:
                        self.log.error('Exception on _setWidgetToolTips: '
                                       '%s', e)
                else:
                    self.log.error('_setWidgetToolTips, what is happening?')

        DEBUG_MISSING_TOOLTIPS = False

        #TODO Evaluate how to integrate this code
        if DEBUG_MISSING_TOOLTIPS:
            for p in UI_PARS:
                if p not in FOUND_PARS and p not in NOT_FOUND_PARS:
                    MISSING_TOOLTIPS.append(p)
            print('\n\nfound:', FOUND_PARS)
            print('\n\nnot found:', NOT_FOUND_PARS)
            print('\n\nmissing:', MISSING_TOOLTIPS)

    @loggingInfo
    def createTableWidget(self, column_names):
        widget = QtWidgets.QWidget()
        table_widget = QtWidgets.QTableWidget(widget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(218, 224, 234))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(
            QtGui.QPalette.Active,
            QtGui.QPalette.AlternateBase,
            brush)
        brush = QtGui.QBrush(QtGui.QColor(218, 224, 234))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(
            QtGui.QPalette.Inactive,
            QtGui.QPalette.AlternateBase,
            brush)
        brush = QtGui.QBrush(QtGui.QColor(218, 224, 234))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(
            QtGui.QPalette.Disabled,
            QtGui.QPalette.AlternateBase,
            brush)
        palette.setColor(QtGui.QPalette.Text, QtGui.QColor(Qt.Qt.black))
        table_widget.setPalette(palette)
        table_widget.setAlternatingRowColors(True)
        table_widget.setSelectionMode(
            QtWidgets.QAbstractItemView.NoSelection)
        table_widget.setSelectionBehavior(
            QtWidgets.QAbstractItemView.SelectItems)
        table_widget.setGridStyle(QtCore.Qt.SolidLine)
        table_widget.horizontalHeader().setStretchLastSection(True)

        columns = len(column_names)
        table_widget.clear()
        table_widget.setColumnCount(columns)
        table_widget.setRowCount(0)

        for i in range(columns):
            item = QtWidgets.QTableWidgetItem()
            item.setText(column_names[i])
            table_widget.setHorizontalHeaderItem(i, item)

        vboxlayout = QtWidgets.QVBoxLayout(widget)
        vboxlayout.setContentsMargins(9, 9, 9, 9)
        vboxlayout.setSpacing(6)
        vboxlayout.addWidget(table_widget)
        widget.setLayout(vboxlayout)

        return (widget, table_widget)

    @loggingInfo
    def _addItemToTable(self, section, row, column, text, editable):
        pass

    @loggingInfo
    def _addWidgetToTable(self, section, row, column,
                          widget_type, min, max, unknownTab=False):
        pass

    @loggingInfo
    def setDescription(self):
        driver = self.icepap_driver
        signature = self.icepap_driver.current_cfg.signature
        desc_cfg_system = self.icepap_driver.icepapsystem_name
        desc_cfg_crate = self.icepap_driver.cratenr
        desc_cfg_addr = self.icepap_driver.addr
        desc_cfg_name = self.icepap_driver.name
        desc_cfg_version = self.icepap_driver.current_cfg.getParameter(
            'VER', True)
        hwversion = IcepapsManager().readIcepapParameters(
            desc_cfg_system, desc_cfg_addr, [['VER', 'PCB']])
        cfg_db = ConfigManager().config["database"]["database"]
        desc_cfg_hwversion = hwversion[0][1]
        desc_cfg_dbhost = ConfigManager().config["database"]["server"]
        if cfg_db == 'sqlite':
            desc_cfg_dbhost = ConfigManager().config["database"]["folder"]
        desc_cfg_date = 'NO_DATE'
        desc_cfg_user = 'NO_USER'
        desc_cfg_host = 'NO_HOST'
        if signature:
            try:
                aux = signature.split('_')
                desc_cfg_host = aux[0]
                # As from version 1.17, the signature includes the username
                user_and_host = aux[0]
                aux2 = user_and_host.split('@')
                if len(aux2) > 1:
                    desc_cfg_user = aux2[0]
                    desc_cfg_host = aux2[1]
                hex_epoch = aux[1]
                try:
                    desc_cfg_date = datetime.datetime.fromtimestamp(
                        int(hex_epoch, 16)).ctime()
                    # AS OF VERSION 1.20, SIGNATURE HAS NOT HEX TIMESTAMP BUT A
                    # MORE READABLE ONE
                except BaseException:
                    desc_cfg_date = datetime.datetime.strptime(
                        aux[1] + '_' + aux[2], '%Y/%m/%d_%H:%M:%S').ctime()

            except Exception:
                msg = 'Not standard signature of driver ' + \
                    str(desc_cfg_addr) + \
                    '.\nIt does not match (user@host_DATE).\nValue is:' + \
                    str(signature)
                MessageDialogs.showWarningMessage(
                    self, "Not standard signature", msg)
        else:
            signature = None

        self.ui.dscDriverName.setText(driver.name)
        self.ui.dscActive.setText(
            driver.current_cfg.getParameter(
                str('ACTIVE')))
        if signature is None:
            self.ui.dscSignature.setText('NO_CONFIG')
        else:
            self.ui.dscSignature.setText(desc_cfg_date)
            html_signature = '<HTML><BODY>'
            html_signature += '<H2>More info:</H2>\n'
            html_signature += '<TABLE>\n'
            html_signature += '<TR><H2>%s/%s/%s</H2></TR>\n' % (
                desc_cfg_system, desc_cfg_crate, desc_cfg_addr)
            html_signature += \
                '<TR><TD> </TD><TD><B>%s</B></TD><TD>%s</TD></TR>\n' % (
                    'Axis name', desc_cfg_name)
            html_signature += \
                '<TR><TD> </TD><TD><B>%s</B></TD><TD>%s</TD></TR>\n' % (
                    'Firmware version', desc_cfg_version)
            html_signature += \
                '<TR><TD> </TD><TD><B>%s</B></TD><TD>%s</TD></TR>\n' % (
                    'Hardware version', desc_cfg_hwversion)
            html_signature += \
                '<TR><TD> </TD><TD><B>%s</B></TD><TD>%s</TD></TR>\n' % (
                    'Database host', desc_cfg_dbhost)
            html_signature += \
                '<TR><TD> </TD><TD><B>%s</B></TD><TD>%s</TD></TR>\n' % (
                    'Last saved config', desc_cfg_date)
            html_signature += \
                '<TR><TD> </TD><TD><B>%s</B></TD><TD>%s</TD></TR>\n' % (
                    'User', desc_cfg_user)
            html_signature += \
                '<TR><TD> </TD><TD><B>%s</B></TD><TD>%s</TD></TR>\n' % (
                    'Host', desc_cfg_host)
            html_signature += \
                '<TR><TD> </TD><TD><B>%s</B></TD><TD>%s</TD></TR>\n' % (
                    'Raw', signature)
            html_signature += '</TABLE></BODY></HTML>\n'
            self.ui.dscSignature.setToolTip(html_signature)

        if self.ui.dscActive.text() == 'YES':
            self.ui.frame_description.setPalette(self.qframe_lightblue_palette)
        else:
            self.ui.frame_description.setPalette(self.qframe_salmon_palette)

    @loggingInfo
    def fillData(self, icepap_driver):
        """ TO-DO STORM review"""
        QtWidgets.QApplication.instance().setOverrideCursor(
            QtGui.QCursor(QtCore.Qt.WaitCursor))
        self.icepap_driver = icepap_driver
        self._disconnectHighlighting()

        self.widgets_modified = []
        self.tabs_modified = {}
        self.tabs_configPending = {}

        driver_key = self.icepap_driver.icepapsystem_name + \
            "_" + str(self.icepap_driver.addr)
        if driver_key in self.saveConfigPending:
            self.saveConfigPending.remove(driver_key)
        dbIcepapSystem = StormManager().getIcepapSystem(
            self.icepap_driver.icepapsystem_name)
        self.dbStartupConfig = dbIcepapSystem.getDriver(
            self.icepap_driver.addr, in_memory=False).startup_cfg
        self.icepap_driver.startup_cfg = self.dbStartupConfig

        unknownParams = False
        self.unknown_table_widget.clear()
        self.unknown_table_widget.setRowCount(0)

        for name, value in icepap_driver.current_cfg.toList():
            if name in self.param_to_widgets:
                widgets = self.param_to_widgets.get(name)
                self._setWidgetsValue(widgets, value)
            elif name in ['IPAPNAME', 'VER', 'ID']:
                # THESE PARAMS DO NOT COME FROM THE FIRMWARE
                # HARDCODED DRIVER NAME
                if name == 'IPAPNAME':
                    self._setWidgetsValue(
                        self.param_to_widgets.get('DriverName'), value)
                    self.icepap_driver.setName(value)
                    # WE SHOULD GET THE TREE NODE AND UPDATE IT'S LABEL
                    # WE SHOULD ALSO SET THE DRIVER NAME
                    modelindex = self._mainwin.ui.treeView.currentIndex()
                    drivernode = self._mainwin._tree_model.item(modelindex)
                    label = str(self.icepap_driver.addr) + \
                        " " + self.icepap_driver.name
                    if drivernode is not None:
                        drivernode.changeLabel([label])
            else:
                unknownParams = True
                self.addUnknownWidget(name, value)

        unknown_index = self.ui.tabWidget.indexOf(self.unknown_tab)

        if unknownParams:
            if unknown_index == -1:
                unknown_index = self.ui.tabWidget.count()
                self.ui.tabWidget.insertTab(
                    unknown_index, self.unknown_tab, "Unknown")
        elif unknown_index != -1:
            self.ui.tabWidget.removeTab(unknown_index)

        self.highlightTabs()

        self.startTesting()

        if self.ui.historicWidget.isVisible():
            self.ui.historicWidget.fillData(self.icepap_driver)

        self.ui.tabWidget.setCurrentIndex(self.lastTabSelected)
        self.setDescription()
        self._connectHighlighting()

        # ALWAYS PUT THE DRIVER IN CONFIG MODE
        # It may happen that the driver is in PROG MODE

        # NOT VALID ANY MORE (SINCE VERSION 1.23 ONLY IN CONFIG IF NECESSARY
        # mode = self._manager.startConfiguringDriver(self.icepap_driver)
        mode = self.icepap_driver.mode
        if mode != Mode.PROG:
            self.ui.tabWidget.setEnabled(True)
        else:
            # MAY BE ALSO GOOD FOR THE SHUTTER MODE
            MessageDialogs.showErrorMessage(
                None,
                'Start configuring driver',
                'It is not possible to configure the driver\n'
                'while it is in mode PROG.')
            self.ui.tabWidget.setEnabled(False)
        QtWidgets.QApplication.instance().restoreOverrideCursor()

    @loggingInfo
    def addUnknownWidget(self, param_name, param_value):
        # TODO Review this code
        cfginfo = IcepapsManager().icepap_cfginfos[
            self.icepap_driver.icepapsystem_name][
            self.icepap_driver.addr].get(param_name)

        if cfginfo is not None:
            row = self.unknown_table_widget.rowCount()
            self.unknown_table_widget.insertRow(row)

            widget = None
            if len(cfginfo) > 0:
                if "INTEGER" == cfginfo[0]:
                    widget = QtWidgets.QSpinBox()
                    widget.setMaximum(999999999)
                    widget.setMinimum(-999999999)
                elif "FLOAT" == cfginfo[0]:
                    widget = QtWidgets.QDoubleSpinBox()
                    widget.setMaximum(999999999)
                    widget.setMinimum(-999999999)
                elif cfginfo[0].startswith("["):
                    param_tooltip = "FLAGS:"
                    for flag in cfginfo:
                        # MORE EFFORT THAN NEEDED BUT...
                        flag.replace("[", "")
                        flag.replace("]", "")
                        param_tooltip += " " + flag
                    widget = QtWidgets.QLineEdit()
                    widget.setToolTip(param_tooltip)
                elif cfginfo[0].startswith("STRING"):
                    widget = QtWidgets.QLineEdit()
                    param_tooltip = str(cfginfo)
                    widget.setToolTip(param_tooltip)
                else:
                    widget = QtWidgets.QComboBox()
                    param_tooltip = str(cfginfo)
                    widget.setToolTip(param_tooltip)

            if widget is not None:
                widget.param = param_name
                widget.isCommand = False
                widget.tab_index = self.ui.tabWidget.indexOf(self.unknown_tab)
                param_item = QtWidgets.QTableWidgetItem()
                param_item.setText(param_name)
                param_item.setFlags(Qt.Qt.ItemIsSelectable)
                self.unknown_table_widget.setItem(row, 0, param_item)
                value_item = QtWidgets.QTableWidgetItem()
                value_item.setText(str(param_value))
                value_item.setFlags(Qt.Qt.ItemIsSelectable)
                self.unknown_table_widget.setItem(row, 1, value_item)

                self.unknown_table_widget.setCellWidget(row, 2, widget)

                self.param_to_unknown_widgets[param_name] = widget

                self._setWidgetsValue([widget], param_value)

                self._connectWidgetToSignalMap(widget)

    @loggingInfo
    def checkSaveConfigPending(self):
        # THIS METHOD IS CALLED WHEN THE USER WANTED TO
        if self.icepap_driver is not None:
            driver_key = self.icepap_driver.icepapsystem_name + \
                "_" + str(self.icepap_driver.addr)
            if driver_key in self.saveConfigPending:
                # IT SHOULD STILL BE EN CONFIG MODE BECAUSE SOME VALUES
                # MISMATCH FROM DATABASE
                return True
            else:
                self._manager.endConfiguringDriver(self.icepap_driver)
                return False

    @loggingInfo
    def _connectWidgetToSignalMap(self, widget):
        self.signalMapper.setMapping(widget, widget)
        if isinstance(widget, QtWidgets.QDoubleSpinBox) or isinstance(
                widget, QtWidgets.QSpinBox):
            widget.valueChanged.connect(self.signalMapper.map)
        elif isinstance(widget, QtWidgets.QCheckBox):
            widget.stateChanged.connect(self.signalMapper.map)
        elif isinstance(widget, QtWidgets.QComboBox):
            widget.currentIndexChanged.connect(self.signalMapper.map)
        elif isinstance(widget, QtWidgets.QLineEdit):
            widget.textChanged.connect(self.signalMapper.map)

    @loggingInfo
    def _setWidgetsValue(self, widgets, value, set_default=True):
        # THE CMD WIDGETS RIGHT NOW ARE ONLY QCOMBOBOXES SO THE SPECIAL CODE IS
        # JUST IN THAT ELIF SECTION
        for widget in widgets:
            try:
                system_name = self.icepap_driver.icepapsystem_name
                driver_addr = self.icepap_driver.addr
                if isinstance(widget, QtWidgets.QDoubleSpinBox) or isinstance(
                        widget, QtWidgets.QSpinBox):
                    widget.setValue(float(value))
                    if set_default:
                        widget.defaultvalue = widget.value()
                elif isinstance(widget, QtWidgets.QCheckBox):
                    state = (value == "1" or value == "YES")
                    widget.setChecked(state)
                    if set_default:
                        widget.defaultvalue = value
                elif isinstance(widget, QtWidgets.QComboBox):
                    widget.clear()
                    param = widget.param
                    controller = IcepapsManager()
                    driver_cfginfo = \
                        controller.icepap_cfginfos[system_name][driver_addr]
                    options = driver_cfginfo.get(param)
                    if options is not None:
                        for option in options:
                            widget.addItem(str(option))
                        # 20130412 IT SEEMS THAT INDEXER COMMAND SHOULD HAVE
                        #          ALSO LINKED AS AN OPTION, BUT IT IS NOT
                        #          PROVIDED BY CFGINFO... I WILL ADD IT
                        #          MANUALLY BUT IT MAY BE REMOVED LATER
                        # 20140219 IT SEEMS _NOW_ THAT IS NOT NEEDED ANY MORE
                        # widget.addItem('LINKED')

                    # SPECIAL CASE TO THE TEST WIDGETS
                    if widget.isCommand:
                        if controller.getDriverActiveStatus(
                                system_name, driver_addr) == "NO":
                            widget.setEnabled(False)
                        else:
                            widget.setEnabled(True)
                            # SHOULD RETRIEVE THE VALUE FROM THE DRIVER'S
                            # COMMAND
                            if param == 'AUXPS':
                                # THIS VALUE DOES NOT COME IN THE CONFIGURATION
                                pass
                            elif param == 'CSWITCH':
                                # THIS VALUE DOES NOT COME NEITHER IN THE
                                # CONFIGUARTION
                                self.log.error('_setWidgetsValue '
                                               'eo....cswitch')
                                pass
                            elif param == 'INDEXER':
                                values = controller.readIcepapParameters(
                                    system_name, driver_addr, ['INDEXER'])
                                indexer_answer = values[0]
                                value = indexer_answer[1]
                                # 20140219 IT SEEMS _NOW_ THAT LINKED IS STILL
                                # NEEDED
                                widget.addItem('LINKED')
                                widget.setEnabled(False)
                            elif param.startswith('INFOA'):
                                values = controller.readIcepapParameters(
                                    system_name, driver_addr, ['INFOA'])
                                infoa_values = values[0][1].split()
                                if param.endswith('SRC'):
                                    value = infoa_values[0]
                                else:
                                    value = infoa_values[1]
                            elif param.startswith('INFOB'):
                                values = controller.readIcepapParameters(
                                    system_name, driver_addr, ['INFOB'])
                                infob_values = values[0][1].split()
                                if param.endswith('SRC'):
                                    value = infob_values[0]
                                else:
                                    value = infob_values[1]
                            elif param.startswith('INFOC'):
                                values = controller.readIcepapParameters(
                                    system_name, driver_addr, ['INFOC'])
                                infoc_values = values[0][1].split()
                                if param.endswith('SRC'):
                                    value = infoc_values[0]
                                else:
                                    value = infoc_values[1]

                    widget.setCurrentIndex(
                        widget.findText(
                            str(value),
                            QtCore.Qt.MatchFixedString))
                    if set_default:
                        widget.defaultvalue = str(value)
                elif isinstance(widget, QtWidgets.QLineEdit):
                    widget.setText(str(value))
                    if set_default:
                        widget.defaultvalue = str(value)
                elif isinstance(widget, QtWidgets.QLabel):
                    widget.setText(str(value))
                elif isinstance(widget, QtWidgets.QFrame):
                    # WE NOW HAVE TO ITERATE THROUGOUT ALL THE CHECKBOXES:
                    if set_default:
                        widget.defaultvalue = value
                    regexp = QtCore.QRegExp('^' + widget.objectName() + "_")
                    for w in self.ui.tabWidget.findChildren(
                            QtWidgets.QWidget, regexp):
                        w_param = w.objectName().split("_")[1]
                        checked = False
                        if w_param in value:
                            checked = True
                        w.setChecked(checked)

                self.highlightWidget(widget)
            except Exception as e:
                self.log.error("_setWidgetValue %s error:", value, e)

    @loggingInfo
    def _getWidgetValue(self, widget):
        try:
            if isinstance(widget, QtWidgets.QDoubleSpinBox) or isinstance(
                    widget, QtWidgets.QSpinBox):
                return widget.value()
            elif isinstance(widget, QtWidgets.QCheckBox):
                if widget.isChecked():
                    return "YES"
                else:
                    return "NO"
            elif isinstance(widget, QtWidgets.QComboBox):
                return str(widget.currentText())
            elif isinstance(widget, QtWidgets.QLineEdit):
                # return str(widget.text()).upper()
                # FIX NON-ASCII ISSUES:
                text = str(widget.text())
                if not all(ord(c) < 128 for c in text):
                    text = repr(text)
                return text
            elif isinstance(widget, QtWidgets.QFrame):
                regexp = QtCore.QRegExp('^' + widget.objectName() + "_")
                flags_value = []
                for w in self.ui.tabWidget.findChildren(QtWidgets.QWidget,
                                                        regexp):
                    w_param_str = str(w.objectName())
                    w_param = "_".join(w_param_str.split("_")[1:])
                    if w.isChecked():
                        flags_value.append(w_param)
                return ' '.join(flags_value)
        except Exception as e:
            self.log.error("error in _getWidgetValue: %s", e)

    @loggingInfo
    def addNewCfg(self, cfg):
        QtWidgets.QApplication.instance().setOverrideCursor(
            QtGui.QCursor(QtCore.Qt.WaitCursor))
        for name, value in cfg.toList():
            if name in self.param_to_widgets:
                widgets = self.param_to_widgets.get(name)
                self._setWidgetsValue(widgets, value, set_default=False)
            elif name == 'IPAPNAME':
                self._setWidgetsValue(self.param_to_widgets.get(
                    'DriverName'), value, set_default=False)
        self.highlightTabs()
        QtWidgets.QApplication.instance().restoreOverrideCursor()

    @loggingInfo
    def _getText(self, node):
        rc = ""
        if node is not None:
            rc = str(node.data)
        return rc

    @loggingInfo
    def tabWidget_currentChanged(self, index):
        self.lastTabSelected = index
        self.highlightTabs()

    @loggingInfo
    def highlightTabs(self):
        tab_bar = self.ui.tabWidget.tabBar()
        for index in range(self.ui.tabWidget.count()):
            if index in self.tabs_modified:
                tab_bar.setTabIcon(index, QtGui.QIcon(
                    ":/icons/icons/ipapdrivermodified.png"))
            elif index in self.tabs_configPending:
                tab_bar.setTabIcon(index, QtGui.QIcon(
                    ":/icons/icons/ipapdrivercfg.png"))
            else:
                tab_bar.setTabIcon(index, QtGui.QIcon(""))

    @loggingInfo
    def btnSendCfg_on_click(self, checked=False, skip_fillData=False):
        if len(self.widgets_modified) == 0:
            return True
        self._mainwin.ui.actionHistoricCfg.setChecked(False)
        self.hideHistoricWidget()
        new_values = []
        new_command_values = []
        for widget in self.widgets_modified:
            # SPECIAL CASE TO THE DRIVER NAME THAT IS NOT A CONFIG PARAMETER
            # BUT IT IS TREATED AS SO, IT IS A COMMAND 'NAME' BUT IT IS STORED
            # IN THE DATABASE AS A CONFIGURATION PARAMETER
            param = widget.param
            value = self._getWidgetValue(widget)
            if not widget.isCommand:
                if param == 'DriverName':
                    param = 'IPAPNAME'
                new_values.append([param, value])
            else:
                # SINCE THE COMMAND PARAMS ARE NOT THE SAME AS THE CFG PARAMS
                # WE NEED A WORK-AROUND HERE AND FOR INFO COMMANDS ALWAYS
                # PROVIDE SRC AND POL IF BOTH SRC AND POL ARE MODIFIED,
                # WE WILL SENT TWICE THE COMMAND PAIR
                if widget.param.startswith('INFOA'):
                    param = 'INFOA'
                    src_value = None
                    pol_value = None
                    for w in self.param_to_widgets.get('INFOASRC'):
                        if w.isCommand:
                            src_value = self._getWidgetValue(w)
                    for w in self.param_to_widgets.get('INFOAPOL'):
                        if w.isCommand:
                            pol_value = self._getWidgetValue(w)
                    value = src_value + " " + pol_value
                elif widget.param.startswith('INFOB'):
                    param = 'INFOB'
                    src_value = None
                    pol_value = None
                    for w in self.param_to_widgets.get('INFOBSRC'):
                        if w.isCommand:
                            src_value = self._getWidgetValue(w)
                    for w in self.param_to_widgets.get('INFOBPOL'):
                        if w.isCommand:
                            pol_value = self._getWidgetValue(w)
                    value = src_value + " " + pol_value
                elif widget.param.startswith('INFOC'):
                    param = 'INFOC'
                    src_value = None
                    pol_value = None
                    for w in self.param_to_widgets.get('INFOCSRC'):
                        if w.isCommand:
                            src_value = self._getWidgetValue(w)
                    for w in self.param_to_widgets.get('INFOCPOL'):
                        if w.isCommand:
                            pol_value = self._getWidgetValue(w)
                    value = src_value + " " + pol_value
                else:
                    # THIS IS ANOTHER cmd that should be passed...
                    # AND BY DEFAULT DESIGN, param and value should be unique.
                    pass
                new_command_values.append([param, value])

        if len(new_values) > 0:
            setExpertFlag = self._mainwin.ui.actionSetExpertFlag.isChecked()
            send_ok = self._manager.saveValuesInIcepap(
                self.icepap_driver, new_values, expertFlag=setExpertFlag)
            if not send_ok:
                MessageDialogs.showWarningMessage(
                    self, "Driver configuration", "Wrong parameter format")
                return False

        if len(new_command_values) > 0:
            self._manager.writeIcepapParameters(
                self.icepap_driver.icepapsystem_name,
                self.icepap_driver.addr,
                new_command_values)

        if not skip_fillData:
            self.fillData(self.icepap_driver)

        if self.icepap_driver.hasUndoList():
            self.ui.btnUndo.setEnabled(True)
        else:
            self.ui.btnUndo.setEnabled(False)
        return True

    @loggingInfo
    def btnSaveCfg_on_click(self):
        save_ok = True
        if len(self.widgets_modified) > 0:
            save_ok = self.btnSendCfg_on_click(skip_fillData=True)
        if not save_ok:
            MessageDialogs.showWarningMessage(
                self, "Driver configuration",
                "Problems found saving the configuration")
            return
        self._mainwin.actionSaveConfigMth()

    @loggingInfo
    def btnUndo_on_click(self):
        self._manager.undoDriverConfiguration(self.icepap_driver)
        self.fillData(self.icepap_driver)
        if not self.icepap_driver.hasUndoList():
            self.ui.btnUndo.setEnabled(False)

    @loggingInfo
    def btnRestore_on_click(self):
        self.fillData(self.icepap_driver)

    @loggingInfo
    def doImport(self):
        filename = ""
        try:
            folder = ConfigManager().config["icepap"]["configs_folder"]
            fn = QtWidgets.QFileDialog.getOpenFileName(
                self, "Open Config File", folder, "*.xml")
            if fn[0] == '':
                return
            filename = str(fn[0])
            self.fillFileData(filename)
        except Exception as e:
            msg = "Error reading file %s: %s".format(filename, e)
            self.log.warning(msg)
            MessageDialogs.showWarningMessage(self, "File", msg)

    @loggingInfo
    def fillFileData(self, filename):
        QtWidgets.QApplication.instance().setOverrideCursor(
            QtGui.QCursor(QtCore.Qt.WaitCursor))
        self._disconnectHighlighting()
        doc = minidom.parse(filename)
        for param_element in doc.getElementsByTagName("par"):
            param_name = param_element.attributes.get('name').value
            param_name = param_name.strip()
            param_value = param_element.attributes.get('value').value
            param_value = param_value.strip()
            if param_name in self.param_to_widgets:
                widgets = self.param_to_widgets.get(param_name)
                self._setWidgetsValue(widgets, param_value, set_default=False)
            elif param_name == 'IPAPNAME':
                self._setWidgetsValue(self.param_to_widgets.get(
                    'DriverName'), param_value, set_default=False)
            elif param_name in ['ID', 'VER']:
                pass
            else:
                # SINCE UNKNOWN WIDGETS ARE CREATED EVERY TIME, ANOTHER DICT
                # HAS TO BE AVAILABLE FOR THEM
                w = self.param_to_unknown_widgets.get(param_name)
                self._setWidgetsValue([w], param_value, set_default=False)

        self.highlightTabs()
        self._connectHighlighting()
        QtWidgets.QApplication.instance().restoreOverrideCursor()

    @loggingInfo
    def doExport(self):
        folder = ConfigManager().config["icepap"]["configs_folder"]
        fn = QtWidgets.QFileDialog.getSaveFileName(
            self, "Save Config File", folder, "*.xml")
        if fn[0] == '':
            return
        filename = str(fn[0])
        if filename.find('.xml') == -1:
            filename = filename + '.xml'
        self.exportToFile(filename)

    @loggingInfo
    def exportToFile(self, filename):
        with open(filename, 'w') as output:
            newdoc = self.getXmlData()
            data = newdoc.toprettyxml()
            output.write(data)

    @loggingInfo
    def getXmlData(self):
        doc = getDOMImplementation().createDocument(None, "Driver", None)
        dbIcepapSystem = StormManager().getIcepapSystem(
            self.icepap_driver.icepapsystem_name)
        dbConfig = dbIcepapSystem.getDriver(
            self.icepap_driver.addr, in_memory=False).startup_cfg
        config_list = sorted(dbConfig.toList())
        for param, value in config_list:
            param_element = doc.createElement("par")
            param_element.setAttribute("name", param)
            param_element.setAttribute("value", str(value))
            doc.documentElement.appendChild(param_element)
        return doc

    @loggingInfo
    def signDriver(self):
        self._disconnectHighlighting()
        self.icepap_driver.signDriver()
        self.fillData(self.icepap_driver)

    @loggingInfo
    def doCopy(self):
        self.temp_file = tempfile.TemporaryFile()
        data = self.getXmlData()
        self.temp_file.write(data.toprettyxml(encoding="utf-8"))
        self.temp_file.flush()
        self.temp_file.seek(0)

    @loggingInfo
    def doPaste(self):
        if self.temp_file is None:
            return
        self.temp_file.seek(0)
        self.fillFileData(self.temp_file)

# ------------------------------  Testing --------------------------------

    @loggingInfo
    def startTesting(self):
        if self.icepap_driver is not None:
            self.inMotion = -1
            self.status = -1
            self.ready = -1
            self.mode = -1
            self.power = -1
            self.updateTestStatus()
            self.refreshTimer.start(1000)

    @loggingInfo
    def stopTesting(self):
        try:
            self.refreshTimer.stop()
            self.setLedsOff()
        except BaseException  as e:
            self.log.error("Unexpected error on stopTesting: %s", e)

    @loggingInfo
    def getMotionValues(self):
        (speed, acc) = self._manager.getDriverMotionValues(
            self.icepap_driver.icepapsystem_name, self.icepap_driver.addr)
        self.ui.txtSpeed.setText(str(speed))
        self.ui.txtAcceleration.setText(str(acc))

    @loggingInfo
    def setMotionValues(self):
        speed = self.ui.txtSpeed.text()
        acc = self.ui.txtAcceleration.text()
        if speed == "":
            speed = "1000"
            self.ui.txtSpeed.setText(speed)
        if acc == "":
            acc = "0.25"
            self.ui.txtAcceleration.setText(acc)
        try:
            self._manager.setDriverMotionValues(
                self.icepap_driver.icepapsystem_name, self.icepap_driver.addr,
                [float(speed), float(acc)])
        except BaseException:
            MessageDialogs.showWarningMessage(
                self, "Driver testing", "Wrong parameter format")

    @loggingInfo
    def setLedsOff(self):
        self.ui.LedError.changeColor(Led.RED)
        self.ui.LedStep.changeColor(Led.YELLOW)
        self.ui.LedHome.changeColor(Led.BLUE)
        self.ui.LedLimitPos.changeColor(Led.ORANGE)
        self.ui.LedLimitNeg.changeColor(Led.GREEN)
        self.ui.LedError.off()
        self.ui.LedStep.off()
        self.ui.LedHome.off()
        self.ui.LedLimitPos.off()
        self.ui.LedLimitNeg.off()
        self.ui.LCDPosition.display(0)

    @loggingInfo
    def disableAllControl(self):
        self.ui.txtSpeed.setEnabled(False)
        self.ui.txtAcceleration.setEnabled(False)
        self.ui.btnGO.setEnabled(False)
        self.ui.btnGORelativeNeg.setEnabled(False)
        self.ui.btnGORelativePos.setEnabled(False)
        self.ui.sliderJog.setEnabled(False)
        self.ui.btnEnable.setEnabled(False)
        self.ui.btnStopMotor.setEnabled(False)

    @loggingInfo
    def enableAllControl(self):
        self.ui.txtSpeed.setEnabled(True)
        self.ui.txtAcceleration.setEnabled(True)

        self.ui.btnGORelativeNeg.setEnabled(True)
        self.ui.btnGORelativePos.setEnabled(True)
        if self.mode == 0:
            self.ui.btnGO.setEnabled(True)
            self.ui.sliderJog.setEnabled(True)
        else:
            self.ui.btnGO.setEnabled(False)
            self.ui.sliderJog.setEnabled(False)
        self.ui.btnEnable.setEnabled(True)
        self.ui.btnStopMotor.setEnabled(True)

    @loggingInfo
    def updateTestStatus(self):
        pos_sel = str(self.ui.cb_pos_sel.currentText())
        enc_sel = str(self.ui.cb_enc_sel.currentText())
        (status, power, position) = self._manager.getDriverTestStatus(
            self.icepap_driver.icepapsystem_name, self.icepap_driver.addr,
            pos_sel, enc_sel)

        axis_state = State(status)
        disabled = axis_state.get_disable_code()
        # TODO: use boolean instead of integers
        moving = int(axis_state.is_moving())
        ready = int(axis_state.is_ready())
        mode = axis_state.get_mode_code()
        if self.inMotion != moving:
            if moving == 1:
                self.refreshTimer.setInterval(700)
                self.ui.LedStep.on()
            elif moving == -1:
                self.refreshTimer.stop()
            else:
                self.refreshTimer.setInterval(1000)
                self.ui.LedStep.off()
        self.inMotion = moving

        if self.status != disabled or self.mode != mode or \
                self.power != power or self.ready != ready:
            if disabled == 0:
                if power:
                    self.ui.LedError.changeColor(Led.GREEN)
                    self.ui.LedError.on()
                    self.ui.btnEnable.setEnabled(True)
                    self.getMotionValues()
                    self.mode = mode
                    self.enableAllControl()
                    self.ui.btnEnable.setText("OFF")
                    self.ui.btnEnable.setChecked(True)
                else:
                    self.ui.btnEnable.setEnabled(True)
                    self.ui.btnEnable.setText("ON")
                    self.ui.btnEnable.setChecked(False)
                    self.ui.LedError.changeColor(Led.RED)
                    self.ui.LedError.on()
            elif disabled == 1:
                # driver is not active disable motion and enable
                self.disableAllControl()
                self.ui.LedError.changeColor(Led.RED)
                self.ui.LedError.on()
            else:
                self.ui.btnEnable.setEnabled(True)
                self.ui.btnEnable.setText("ON")
                self.ui.btnEnable.setChecked(False)
                self.ui.LedError.changeColor(Led.RED)
                self.ui.LedError.on()

        self.status = disabled
        self.ready = ready
        self.power = power

        if axis_state.is_inhome():
            self.ui.LedHome.on()
        else:
            self.ui.LedHome.off()

        lower = axis_state.is_limit_negative()
        upper = axis_state.is_limit_positive()
        if lower:
            self.ui.LedLimitNeg.on()
        else:
            self.ui.LedLimitNeg.off()

        if upper:
            self.ui.LedLimitPos.on()
        else:
            self.ui.LedLimitPos.off()

        # read position and encoder

        self.ui.LCDPosition.display(position[0])
        self.ui.LCDPositionTest.display(position[0])
        self.ui.LCDEncoder.display(position[1])

    @loggingInfo
    def btnGO_on_click(self):
        new_position = self.ui.txtMvAbsolute.text()
        try:
            new_position = int(new_position)
            self._manager.moveDriverAbsolute(
                self.icepap_driver.icepapsystem_name,
                self.icepap_driver.addr,
                new_position)
        except BaseException:
            MessageDialogs.showWarningMessage(
                self, "Driver testing", "Wrong parameter format")

    @loggingInfo
    def btnGORelativePos_on_click(self):
        distance = self.ui.txtGORelative.text()
        try:
            distance = int(distance)
            steps = +distance
            self._manager.moveDriver(
                self.icepap_driver.icepapsystem_name,
                self.icepap_driver.addr,
                steps)

        except BaseException:
            MessageDialogs.showWarningMessage(
                self, "Driver testing", "Wrong parameter format")

    @loggingInfo
    def btnGORelativeNeg_on_click(self):
        distance = self.ui.txtGORelative.text()
        try:
            distance = int(distance)
            steps = -distance
            self._manager.moveDriver(
                self.icepap_driver.icepapsystem_name,
                self.icepap_driver.addr,
                steps)

        except BaseException:
            MessageDialogs.showWarningMessage(
                self, "Driver testing", "Wrong parameter format")

    @loggingInfo
    def btnStopMotor_on_click(self):
        self._manager.stopDriver(
            self.icepap_driver.icepapsystem_name,
            self.icepap_driver.addr)
        self.ui.sliderJog.setValue(0)

    @loggingInfo
    def btnBlink_on_press(self):
        secs = 600
        if self.ui.btnBlink.isChecked():
            secs = 0
        self._manager.blinkDriver(
            self.icepap_driver.icepapsystem_name,
            self.icepap_driver.addr,
            secs)

    @loggingInfo
    def sliderChanged(self, div):
        if self.ui.sliderJog.isSliderDown() or not self.sliderTimer.isActive():
            self.startJogging(div)

    @loggingInfo
    def startJogging(self, div):
        if div != 0:
            if not self.ui.btnEnable.isChecked():
                self._manager.enableDriver(
                    self.icepap_driver.icepapsystem_name,
                    self.icepap_driver.addr)
            speed = float(self.ui.txtSpeed.text())
            factor = (self.ui.sliderJog.maximum() - abs(div)) + 1
            speed = int(speed / factor)
            if div < 0:
                speed = -1 * speed
            try:
                QtWidgets.QToolTip.showText(
                    self.cursor().pos(), str(speed), self.ui.sliderJog)
                self._manager.jogDriver(
                    self.icepap_driver.icepapsystem_name,
                    self.icepap_driver.addr,
                    speed)
            except Exception as e:
                MessageDialogs.showWarningMessage(
                    self, "Jog Driver",
                    "Error while trying to jog:\n" + str(e))
                self.ui.sliderJog.setValue(0)
        else:
            self.stopJogging()

    @loggingInfo
    def stopJogging(self):
        self._manager.stopDriver(
            self.icepap_driver.icepapsystem_name,
            self.icepap_driver.addr)
        self.ui.sliderJog.setValue(0)
        self.sliderTimer.start()

    @loggingInfo
    def resetSlider(self):
        value = self.ui.sliderJog.value()
        if value == 0:
            self.sliderTimer.stop()
        elif value > 0:
            self.ui.sliderJog.triggerAction(
                QtWidgets.QSlider.SliderSingleStepSub)
        else:
            self.ui.sliderJog.triggerAction(
                QtWidgets.QSlider.SliderSingleStepAdd)

    @loggingInfo
    def setPosition(self):
        pos_sel = str(self.ui.cb_pos_sel.currentText())
        try:
            position = int(self.ui.txtPos.text())
            self._manager.setDriverPosition(
                self.icepap_driver.icepapsystem_name,
                self.icepap_driver.addr,
                pos_sel,
                position)
        except BaseException as e:
            self.log.error("Unexpected error on setPosition: %s", e)
            MessageDialogs.showWarningMessage(
                self, "Set driver position", "Wrong parameter format")

    @loggingInfo
    def setEncoder(self):
        enc_sel = str(self.ui.cb_enc_sel.currentText())
        try:
            position = int(self.ui.txtEnc.text())
            self._manager.setDriverEncoder(
                self.icepap_driver.icepapsystem_name,
                self.icepap_driver.addr,
                enc_sel,
                position)
        except BaseException as e:
            self.log.error("Unexpected error on setEncoder: %s", e)
            MessageDialogs.showWarningMessage(
                self, "Set driver encoderposition", "Wrong parameter format")

    @loggingInfo
    def endisDriver(self, bool):
        if bool:
            self.ui.btnEnable.setText("OFF")
            self._manager.enableDriver(
                self.icepap_driver.icepapsystem_name,
                self.icepap_driver.addr)
        else:
            self.ui.btnEnable.setText("ON")
            self._manager.disableDriver(
                self.icepap_driver.icepapsystem_name,
                self.icepap_driver.addr)

    @loggingInfo
    def changeSwitchesSetup(self, mode):
        return
        # SHOULD CHANGE SWITCHES SETUP... HOW? NOT IN CONFIG MODE...

    # ---------------------- Historic Widget -------------------

    @loggingInfo
    def showHistoricWidget(self):
        self.ui.stackedWidget.setCurrentIndex(1)
        self.ui.historicWidget.fillData(self.icepap_driver)

    @loggingInfo
    def hideHistoricWidget(self):
        self.ui.stackedWidget.setCurrentIndex(0)

    # ---------------------- Templates Catalog Widget -------------------

    @loggingInfo
    def setTemplateParams(self, template_name, params):
        QtWidgets.QApplication.instance().setOverrideCursor(
            QtGui.QCursor(QtCore.Qt.WaitCursor))
        self._disconnectHighlighting()
        for param in list(params.keys()):
            value = params.get(param)
            if param in self.param_to_widgets:
                widgets = self.param_to_widgets.get(param)
                self._setWidgetsValue(widgets, value, set_default=False)
            elif param == 'IPAPNAME':
                self._setWidgetsValue(self.param_to_widgets.get(
                    'DriverName'), value, set_default=False)
        # SHOULD DO SOMETHING WITH THE TEMPLATE NAME
        self.highlightTabs()
        self._connectHighlighting()
        QtWidgets.QApplication.instance().restoreOverrideCursor()


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)

    w = PageiPapDriver(None, True)
    w.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

__all__ = ['Singleton']


class Singleton(object):
    def __new__(cls, *p, **k):
        if '_the_instance' not in cls.__dict__:
            cls._the_instance = object.__new__(cls)
            cls._the_instance.init(*p)
        return cls._the_instance

    def init(self, *p):
        pass



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from pkg_resources import resource_filename
from storm.locals import Store, create_database
import os
import re
import logging
from .singleton import Singleton
from .configmanager import ConfigManager
from ..helpers import loggingInfo
from .stormmysql import MySQL

__all__ = ['StormManager']


class StormManager(Singleton):
    log = logging.getLogger('{}.StormManager'.format(__name__))

    def __init__(self):
        pass

    @loggingInfo
    def init(self, *args):
        self.dbOK = False
        self.openDB()

    @loggingInfo
    def reset(self):
        self.closeDB()
        self.openDB()

    @loggingInfo
    def openDB(self):
        try:
            self._config = ConfigManager()
            self.db = self._config.config[self._config.database]["database"]
            create_db = False
            if self.db == self._config.Sqlite:
                folder = self._config.config[self._config.database]["folder"]
                loc = folder + '/icepapcms.db'
                create_db = not os.path.exists(loc)
                if create_db:
                    if not os.path.exists(folder):
                        os.mkdir(folder)
                self._database = create_database("%s:%s" % (self.db, loc))
            else:
                server = self._config.config[self._config.database]["server"]
                user = self._config.config[self._config.database]["user"]
                pwd = self._config.config[self._config.database]["password"]
                scheme = "{}://{}:{}@{}/icepapcms" .format(self.db, user, pwd,
                                                           server)

                if self.db == 'mysql':
                    self._database = MySQL(scheme)
                else:
                    self._database = create_database(scheme)

            self._store = Store(self._database)
            if create_db:
                self.dbOK = self.createSqliteDB()
            else:
                self.dbOK = True
        except Exception as e:
            self.log.error("Unexpected error on openDB: %s", e)
            self.dbOK = False

    @loggingInfo
    def createSqliteDB(self):
        try:
            sql_file = resource_filename('icepapcms.db',
                                         'creates_sqlite.sql')
            with open(sql_file, 'rb') as f:
                sql_script = f.read().decode()
            statements = re.compile(r";[ \t]*$", re.M)

            for statement in statements.split(sql_script):
                # Remove any comments from the file
                statement = re.sub(r"--.*[\n\\Z]", "", statement)
                if statement.strip():
                    create = statement + ";"
                    self._store.execute(create)
            self._store.commit()
            return True
        except Exception as e:
            self.log.error("Unexpected error on createSqliteDB: %s", e)
            return False

    @loggingInfo
    def closeDB(self):
        try:
            if self.dbOK:
                self._store.close()
            return True
        except Exception as e:
            self.log.error("Unexpected error on closeDB:", e)
            self.dbOK = False
            return False

    @loggingInfo
    def store(self, obj):
        self._store.add(obj)

    @loggingInfo
    def remove(self, obj):
        self._store.remove(obj)

    @loggingInfo
    def addIcepapSystem(self, icepap_system):
        try:
            self._store.add(icepap_system)
            self.commitTransaction()
            return True
        except Exception as e:
            self.log.error("some exception trying to store the icepap system "
                           "%s: %s", icepap_system, e)
            return False

    @loggingInfo
    def deleteLocation(self, location):
        if self.db == self._config.Sqlite:
            for system in location.systems:
                self.deleteIcepapSystem(system)
        self._store.remove(location)
        self.commitTransaction()

    @loggingInfo
    def deleteIcepapSystem(self, icepap_system):
        if self.db == self._config.Sqlite:
            for driver in icepap_system.drivers:
                self.deleteDriver(driver)
        self._store.remove(icepap_system)
        self.commitTransaction()

    @loggingInfo
    def deleteDriver(self, driver):

        for cfg in driver.historic_cfgs:
            for par in cfg.parameters:
                self._store.remove(par)
            self._store.remove(cfg)
        self._store.remove(driver)
        self.commitTransaction()

    @loggingInfo
    def getAllLocations(self):
        try:
            locations = self._store.find(Location)
            location_dict = {}
            for location in locations:
                location_dict[location.name] = location
            return location_dict
        except Exception as e:
            self.log.error("Unexpected error on getAllLocations: %s", e)
            return {}

    @loggingInfo
    def getLocation(self, name):
        return self._store.get(Location, name)

    @loggingInfo
    def getIcepapSystem(self, icepap_name):
        return self._store.get(IcepapSystem, icepap_name)

    @loggingInfo
    def existsDriver(self, mydriver, id):

        drivers = self._store.find(
            IcepapDriver, IcepapDriver.addr == IcepapDriverCfg.driver_addr,
            IcepapDriverCfg.id == CfgParameter.cfg_id,
            CfgParameter.name == str("ID"), CfgParameter.value == id)
        if drivers:
            for driver in drivers:
                if driver.addr != mydriver.addr:
                    return driver
            return None
        else:
            return None

    @loggingInfo
    def getLocationIcepapSystem(self, location):
        try:
            icepaps = self._store.find(IcepapSystem,
                                       IcepapSystem.location_name == location)
            icepaps.order_by(IcepapSystem.name)
            ipapdict = {}
            for ipap_sys in icepaps:
                ipapdict[ipap_sys.name] = ipap_sys
            return ipapdict
        except Exception as e:
            self.log.error("Unexpected error on getLocationIcepapSystem: "
                           "%s", e)
            return {}

    @loggingInfo
    def rollback(self):
        self._store.rollback()

    @loggingInfo
    def commitTransaction(self):
        try:
            self._store.commit()
            return True
        except Exception:
            return False


# TODO Avoid circular imports
from .icepapsystem import Location, IcepapSystem
from .icepapdriver import IcepapDriver
from .icepapdrivercfg import CfgParameter, IcepapDriverCfg



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtCore, QtGui, uic, QtWidgets
from pkg_resources import resource_filename
import logging
from ..helpers import loggingInfo
from .messagedialogs import MessageDialogs


class HistoricCfgWidget(QtWidgets.QWidget):
    log = logging.getLogger('{}.HistoricCfgWidget'.format(__name__))

    @loggingInfo
    def __init__(self, parent=None):
        QtWidgets.QWidget.__init__(self, parent)
        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'historiccfgwidget.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)
        self.icepap_driver = None
        self.selectedCfg = None
        self.selectedDays = {}

        # Connect Signals
        self.ui.calendarWidget.clicked.connect(self.daySelected)
        self.ui.btnBack.clicked.connect(self.btnBackClicked)
        self.ui.listWidget.currentTextChanged.connect(self.listWidgetChanged)
        self.ui.saveButton.clicked.connect(self.saveButton_on_click)
        self.ui.deleteButton.clicked.connect(self.deleteButton_on_click)

    @loggingInfo
    def setCfgPage(self, pagedriver):
        self.pagedriver = pagedriver

    @loggingInfo
    def fillData(self, driver):
        """ TO-DO STORM review"""
        self.ui.stackedWidget.setCurrentIndex(0)
        self.ui.saveButton.setEnabled(False)
        self.ui.deleteButton.setEnabled(False)

        for date in self.ui.calendarWidget.dateTextFormat():
            self.ui.calendarWidget.setDateTextFormat(
                date, QtGui.QTextCharFormat())
        self.selectedDays = {}
        self.icepap_driver = driver
        self.ui.txtName.setText("")
        self.ui.txtDescription.setText("")
        for cfg in self.icepap_driver.historic_cfgs:
            datetime = cfg.date

            qdate = QtCore.QDate(datetime.year, datetime.month, datetime.day)
            cfgdate = qdate.toPyDate().ctime()
            if cfgdate in self.selectedDays:
                self.selectedDays[cfgdate].append([cfg.date, cfg])
            else:
                format = QtGui.QTextCharFormat()
                format.setBackground(QtGui.QColor(255, 255, 0))
                self.ui.calendarWidget.setDateTextFormat(qdate, format)
                self.selectedDays[cfgdate] = [[cfg.date, cfg]]
        self.daySelected(self.ui.calendarWidget.selectedDate())

    @loggingInfo
    def daySelected(self, qdate):
        date = qdate.toPyDate().ctime()
        if date in self.selectedDays:
            daylist = self.selectedDays[date]
            if len(daylist) == 1:
                self.fillCfgData(daylist[0])
            else:
                self.ui.stackedWidget.setCurrentIndex(1)
                self.fillDayCfgs(daylist)

    @loggingInfo
    def fillDayCfgs(self, cfgs):
        self.ui.listWidget.clear()
        self.listDict = {}
        for date, cfg in cfgs:
            key = date.ctime()
            self.listDict[key] = [date, cfg]
            self.ui.listWidget.addItem(key)

    @loggingInfo
    def listWidgetChanged(self, name):
        name = str(name)
        if name in self.listDict:
            self.fillCfgData(self.listDict[name])

    @loggingInfo
    def fillCfgData(self, cfg):
        self.ui.saveButton.setEnabled(True)
        if self.icepap_driver.current_cfg.name == cfg[1].name:
            self.ui.deleteButton.setEnabled(False)
        else:
            self.ui.deleteButton.setEnabled(True)
        self.selectedCfg = cfg
        self.ui.txtName.setText(cfg[1].name)
        self.ui.txtDescription.setText(str(cfg[1].description))
        self.pagedriver.addNewCfg(self.selectedCfg[1])

    @loggingInfo
    def btnBackClicked(self):
        self.ui.stackedWidget.setCurrentIndex(0)

    @loggingInfo
    def deleteButton_on_click(self):
        if MessageDialogs.showYesNoMessage(
                self, "Historic configuration",
                "Delete selected configuration ?"):
            self.icepap_driver.deleteHistoricCfg(self.selectedCfg[1])
            self.fillData(self.icepap_driver)
        self.ui.txtName.setText("")
        self.ui.txtDescription.clear()
        self.ui.deleteButton.setEnabled(False)

    @loggingInfo
    def saveButton_on_click(self):
        name = str(self.ui.txtName.text())
        desc = str(self.ui.txtDescription.toPlainText())
        if name == "" or not self.selectedCfg:
            MessageDialogs.showWarningMessage(
                self, "Store historic configuration",
                "Fill all required data.")
            return
        if MessageDialogs.showYesNoMessage(
                self, "Store historic configuration",
                "Save configuration information?"):
            self.selectedCfg[1].name = name
            self.selectedCfg[1].description = desc


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)

    w = HistoricCfgWidget(None)
    w.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtWidgets, uic
from pkg_resources import resource_filename
import logging
from ..lib import ConfigManager
from .messagedialogs import MessageDialogs
from ..helpers import loggingInfo

# TODO Change to properties
MYSQL_PORT = 3306
POSTGRES_PORT = 5432


# TODO: Change Debug Level widget to use more levels
class DialogPreferences(QtWidgets.QDialog):
    log = logging.getLogger('{}.DialogPreferences'.format(__name__))

    @loggingInfo
    def __init__(self, parent):
        QtWidgets.QDialog.__init__(self, parent)

        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'dialogpreferences.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)
        self.modal = True
        self.StorageChanged = False
        self.selectedDB = ""

        # Connect Signals
        self.ui.listWidget.itemClicked.connect(self.listWidget_on_click)
        self.ui.btnBrowser.clicked.connect(self.btnBrowse_on_click)
        self.ui.btnLogBrowser.clicked.connect(self.btnLogBrowse_on_click)
        self.ui.btnFirmwareBrowser.clicked.connect(
            self.btnFirmwareBrowse_on_click)
        self.ui.btnConfigsBrowser.clicked.connect(
            self.btnConfigsBrowse_on_click)
        self.ui.btnTemplatesBrowser.clicked.connect(
            self.btnTemplatesBrowse_on_click)
        self.ui.closeButton.clicked.connect(self.closeButton_on_click)
        self.ui.rbmysql.toggled.connect(self.rbMySql_toogled)
        self.ui.rbpostgres.toggled.connect(self.rbPostgres_toogled)
        self.ui.rbsqlite.toggled.connect(self.rbSqlite_toogled)
        self.ui.btnSnapshotsBrowser.clicked.connect(self.snapshots_browser)

        self._config = ConfigManager()
        self.fillConfig()
        self.ui.listWidget.item(0).setSelected(True)
        """ check imports for dbs to disable errors """

    @loggingInfo
    def closeButton_on_click(self):
        if self.checkPreferences():
            self._config.saveConfig()
            self.close()
        else:
            MessageDialogs.showWarningMessage(
                self, "Preferences", "Check configuration parameters")

    @loggingInfo
    def listWidget_on_click(self, item):
        index = self.ui.listWidget.row(item)
        self.ui.stackedWidget.setCurrentIndex(index)

    @loggingInfo
    def rbSqlite_toogled(self, checked):
        if checked:
            self.selectedDB = self._config.Sqlite
            self.ui.gbLocal.setEnabled(True)
            self.ui.gbRemote.setEnabled(False)

    @loggingInfo
    def rbPostgres_toogled(self, checked):
        if checked:
            self.selectedDB = self._config.Postgres
            self.ui.gbLocal.setEnabled(False)
            self.ui.gbRemote.setEnabled(True)
            self.ui.txtPort.setText(str(POSTGRES_PORT))

    @loggingInfo
    def rbMySql_toogled(self, checked):
        if checked:
            self.selectedDB = self._config.MySql
            self.ui.gbLocal.setEnabled(False)
            self.ui.gbRemote.setEnabled(True)
            self.ui.txtPort.setText(str(MYSQL_PORT))

    @loggingInfo
    def btnBrowse_on_click(self):
        current_folder = self._config.config[self._config.database]["folder"]
        fn = QtWidgets.QFileDialog.getExistingDirectory(
            self, "Open Folder", current_folder)
        if fn == '':
            return
        folder = str(fn)
        self.ui.txtLocalFolder.setText(folder)

    @loggingInfo
    def btnLogBrowse_on_click(self):
        current_folder = self._config.config[self._config.icepap]["log_folder"]
        fn = QtWidgets.QFileDialog.getExistingDirectory(
            self, "Open Log Folder", current_folder)
        if fn == '':
            return
        folder = str(fn)
        self.ui.txtLogFolder.setText(folder)

    @loggingInfo
    def btnFirmwareBrowse_on_click(self):
        current_folder = \
            self._config.config[self._config.icepap]["firmware_folder"]
        fn = QtWidgets.QFileDialog.getExistingDirectory(
            self, "Open Firmware Folder", current_folder)
        if fn == '':
            return
        folder = str(fn)
        self.ui.txtFirmwareFolder.setText(folder)

    @loggingInfo
    def btnConfigsBrowse_on_click(self):
        current_folder = \
            self._config.config[self._config.icepap]["configs_folder"]
        fn = QtWidgets.QFileDialog.getExistingDirectory(
            self, "Open Configs Folder", current_folder)
        if fn == '':
            return
        folder = str(fn)
        self.ui.txtConfigsFolder.setText(folder)

    @loggingInfo
    def btnTemplatesBrowse_on_click(self):
        current_folder = \
            self._config.config[self._config.icepap]["templates_folder"]
        fn = QtWidgets.QFileDialog.getExistingDirectory(
            self, "Open Templates Folder", current_folder)
        if fn == '':
            return
        folder = str(fn)
        self.ui.txtTemplatesFolder.setText(folder)

    def snapshots_browser(self):
        current_folder = \
            self._config.config[self._config.icepap]["snapshots_folder"]
        fn = QtWidgets.QFileDialog.getExistingDirectory(
            self, "Open Snapshots Folder", current_folder)
        if fn == '':
            return
        folder = str(fn)
        self.ui.txtSnapshotsFolder.setText(folder)

    @loggingInfo
    def checkDbEngines(self):
        module_errors = ""
        ok_sqlite = True
        try:
            from pysqlite2 import dbapi2 as sqlite
        except ImportError:
            try:
                from sqlite3 import dbapi2 as sqlite
            except ImportError:
                ok_sqlite = False
                module_errors = module_errors + \
                    "Failed to import Sqlite Modules: pysqlite2, sqlite3 " \
                    "modules\n"
        self.ui.rbsqlite.setEnabled(ok_sqlite)

        postgres = True
        try:
            import psycopg2
            import psycopg2.extensions
        except BaseException:
            postgres = False
            module_errors += "Failed to import Postgres modules:psycopg2\n"
        self.ui.rbpostgres.setEnabled(postgres)

        mysql = True
        try:
            import MySQLdb
            import MySQLdb.converters
        except BaseException:
            module_errors += "Failed to import MySQL modules: MySQLdb\n"
            mysql = False
        if module_errors != "":
            module_errors += "Check IcepapCMS user manual to solve these " \
                             "issues"
        self.ui.lblModules.setText(module_errors)
        self.ui.rbmysql.setEnabled(mysql)

    @loggingInfo
    def fillConfig(self):
        ''' storage configuration'''
        self.checkDbEngines()
        db = self._config.config[self._config.database]["database"]
        rb = getattr(self.ui, "rb" + db)
        rb.setChecked(True)
        self.ui.txtLocalFolder.setText(
            self._config.config[self._config.database]["folder"])
        server = self._config.config[self._config.database]["server"]
        server = server.split(':')
        self.ui.txtHost.setText(server[0])
        self.ui.txtPort.setText(server[1])
        user = self._config.config[self._config.database]["user"]
        pwd = self._config.config[self._config.database]["password"]
        self.ui.txtUser.setText(user)
        self.ui.txtPassword.setText(pwd)

        ''' icepap configuration'''
        config = self._config.config[self._config.icepap]
        debug_enabled = config["debug_enabled"]
        debug_level = config["debug_level"]
        log_folder = config["log_folder"]
        firmware_folder = config["firmware_folder"]
        configs_folder = config["configs_folder"]
        templates_folder = config["templates_folder"]
        snapshots_folder = config['snapshots_folder']

        self.ui.chkDebug.setChecked(debug_enabled == str(True))
        self.ui.sbDebugLevel.setValue(int(debug_level))
        self.ui.txtLogFolder.setText(log_folder)
        self.ui.txtFirmwareFolder.setText(firmware_folder)
        self.ui.txtConfigsFolder.setText(configs_folder)
        self.ui.txtTemplatesFolder.setText(templates_folder)
        self.ui.txtSnapshotsFolder.setText(snapshots_folder)

    @loggingInfo
    def checkPreferences(self):
        try:
            ''' Storage Configuration '''
            config_db = self._config.config[self._config.database]
            if config_db["database"] != str(self.selectedDB):
                self.StorageChanged = True
                config_db["database"] = str(self.selectedDB)

            if self.ui.rbsqlite.isChecked():
                local_folder = str(self.ui.txtLocalFolder.text())
                if config_db["folder"] != local_folder:
                    config_db["folder"] = local_folder
                    self.StorageChanged = True
            else:
                host = str(self.ui.txtHost.text()).strip()
                port = str(self.ui.txtPort.text()).strip()
                user = str(self.ui.txtUser.text()).strip()
                pwd = str(self.ui.txtPassword.text()).strip()
                if host == "" or port == "":
                    return False
                remote_server = host + ":" + port
                if config_db["server"] != remote_server:
                    self.StorageChanged = True
                    config_db["server"] = remote_server
                if user != config_db["user"] or pwd != config_db["password"]:
                    self.StorageChanged = True
                    config_db["user"] = user
                    config_db["password"] = pwd

            ''' icepap configuration'''
            config_ipap = self._config.config[self._config.icepap]
            config_ipap["debug_enabled"] = str(self.ui.chkDebug.isChecked())
            config_ipap["debug_level"] = int(self.ui.sbDebugLevel.value())
            config_ipap["log_folder"] = self.ui.txtLogFolder.text()
            config_ipap["firmware_folder"] = self.ui.txtFirmwareFolder.text()
            config_ipap["configs_folder"] = self.ui.txtConfigsFolder.text()
            config_ipap["templates_folder"] = self.ui.txtTemplatesFolder.text()
            config_ipap["snapshots_folder"] = self.ui.txtSnapshotsFolder.text()

            return True
        except BaseException as e:
            self.log.error("Unexpected error on checkPreferences: %s", e)
            return False


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)

    w = DialogPreferences(None)
    w.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


import sys
import os
import webbrowser
from PyQt5 import QtCore, QtGui, Qt, QtWidgets, uic
from pkg_resources import resource_filename, get_distribution, \
    DistributionNotFound
import subprocess
import logging
from ..lib import MainManager, Conflict, ConfigManager, \
    StormManager
from .icepap_treemodel import IcepapTreeModel
from .pageipapdriver import PageiPapDriver
from .pageipapcrate import PageiPapCrate
from .pageipapsystem import PageiPapSystem
from .dialogaddicepap import DialogAddIcepap
from .dialogaddlocation import DialogAddLocation

from .dialogconflictdriver_nonexpert import DialogConflictNonExpert
from .dialogconflictdriver_expert import DialogConflictExpert
from .dialognewdriver import DialogNewDriver

from .dialogpreferences import DialogPreferences
from .dialogipapprogram import DialogIcepapProgram
from .dialogsnapshot import DialogSnapshot
from .ipapconsole import IcepapConsole
from .messagedialogs import MessageDialogs
from .templatescatalogwidget import TemplatesCatalogWidget
from ..helpers import loggingInfo
from ..gui.ldap.login import DialogLdapLogin

__version__ = '3.1.3'


class IcepapApp(QtWidgets.QApplication):
    log = logging.getLogger('{}.IcepapApp'.format(__name__))

    def __init__(self,parent=None):
        QtWidgets.QApplication.__init__(self, [])

    @loggingInfo
    def start(self):
        self.setStyle("plastique")
        splash_pxmap = QtGui.QPixmap(":/logos/icons/IcepapMed.png")
        splash = QtWidgets.QSplashScreen(splash_pxmap)
        splash.show()
        icepapcms = IcepapCMS()
        icepapcms.show()
        splash.finish(icepapcms)
        self.exec_()


class IcepapCMS(QtWidgets.QMainWindow):
    log = logging.getLogger('{}.IcepapCMS'.format(__name__))

    @loggingInfo
    def __init__(self, parent=None):
        QtWidgets.QMainWindow.__init__(self, parent)

        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'icepapcms.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)

        self._config = ConfigManager()

        default_user = 'NotValidated'
        self._config.username = default_user
        if os.name == 'posix':  # this works for linux and macOSX
            self._config.username = os.getenv('USER', default_user)
        elif os.name == 'nt':  # win NT, XP... (and Vista?)
            self._config.username = os.getenv('USERNAME', default_user)

        if self._config.config['ldap']['use']:
            # FORCE AN LDAP LOGIN TO GET CORRECT USER NAMES IN THE DRIVER
            # SIGNATURES
            login = DialogLdapLogin(self, config=self._config.config['ldap'])
            accepted = login.exec()
            if not accepted:
                sys.exit(-1)
            self._config.username = login.username

        self.checkTimer = Qt.QTimer(self)

        self.initGUI()

        self.ui.pageiPapSystem = PageiPapSystem(self)
        self.ui.stackedWidget.addWidget(self.ui.pageiPapSystem)
        self.ui.pageiPapCrate = PageiPapCrate(self)
        self.ui.stackedWidget.addWidget(self.ui.pageiPapCrate)
        self.ui.pageiPapDriver = PageiPapDriver(self)
        self.ui.stackedWidget.addWidget(self.ui.pageiPapDriver)
        self.signalConnections()
        self.refreshTimer = Qt.QTimer(self)
        self.checkTimer.timeout.connect(self.checkIcepapConnection)

    @loggingInfo
    def signalConnections(self):
        self.ui.actionQuit.triggered.connect(self.close)
        self.ui.actionPreferences.triggered.connect(self.actionPreferenceMth)
        self.ui.actionGoNext.triggered.connect(self.actionGoNextMth)
        self.ui.actionTree_Explorer.triggered.connect(
            self.actionTreeExplorerMth)
        self.ui.actionToolbar.triggered.connect(self.actionToolbarMth)
        self.ui.actionGoPrevious.triggered.connect(self.actionGoPreviousMth)
        self.ui.actionGoUp.triggered.connect(self.actionGoUpMth)
        self.ui.actionRefresh.triggered.connect(self.actionRefreshMth)
        self.ui.actionExport.triggered.connect(self.actionExportMth)
        self.ui.actionImport.triggered.connect(self.actionImportMth)
        self.ui.actionConsole.triggered.connect(self.actionConsoleMth)
        self.ui.actionFirmwareUpgrade.triggered.connect(
            self.actionFirmwareUpgradeMth)
        self.ui.actionSaveConfig.triggered.connect(self.actionSaveConfigMth)
        self.ui.actionHistoricCfg.triggered.connect(self.actionHistoricCfgMth)
        self.ui.actionOscilloscope.triggered.connect(self.action_osc_clicked)
        self.ui.actionCopy.triggered.connect(self.actionCopyMth)
        self.ui.actionPaste.triggered.connect(self.actionPasteMth)
        self.ui.actionHelp.triggered.connect(self.actionHelpMth)
        self.ui.actionUser_manual.triggered.connect(self.actionUser_ManualMth)
        self.ui.actionHardware_manual.triggered.connect(
            self.actionHardware_ManualMth)
        self.ui.actionTemplates.triggered.connect(self.actionTemplatesMth)
        self.ui.treeView.clicked.connect(self.treeview_on_click)
        self.ui.treeView.doubleClicked.connect(self.treeview_on_doubleclick)
        self.ui.treeView.setContextMenuPolicy(Qt.Qt.CustomContextMenu)
        self.ui.treeView.customContextMenuRequested.connect(self.__contextMenu)
        self.ui.btnTreeAdd.clicked.connect(self.btnTreeAdd_on_click)
        self.ui.btnTreeRemove.clicked.connect(self.btnTreeRemove_on_click)
        self.ui.actionAddIcepap.triggered.connect(self.btnTreeAdd_on_click)
        self.ui.actionDeleteIcepap.triggered.connect(
            self.btnTreeRemove_on_click)
        self.ui.menuView.aboutToShow.connect(self.menuView_before_show)
        self.ui.cbLocation.activated.connect(self.locationChanged)
        self.ui.actionAddLocation.triggered.connect(self.addLocation)
        self.ui.actionDeleteLocation.triggered.connect(self.deleteLocation)
        self.ui.actionAbout.triggered.connect(self.about)
        self.ui.actionSnapshot.triggered.connect(self.snapshot)

    @loggingInfo
    def initGUI(self):
        self._manager = MainManager(self)
        if not self._manager.dbStatusOK:
            MessageDialogs.showErrorMessage(
                self, "Storage",
                "Error accessing database.\nCheck storage preferences.")

        self.buildLocationCombo()
        self.checkTimer.start(5000)
        self.locationsNext = []
        self.locationsPrevious = []
        self.currentLocation = ""
        self.ui.actionGoNext.setEnabled(False)
        self.ui.actionGoPrevious.setEnabled(False)
        self.ui.actionExport.setEnabled(False)
        self.ui.actionImport.setEnabled(False)
        self.ui.actionHistoricCfg.setEnabled(False)
        self.ui.actionOscilloscope.setEnabled(False)
        self.ui.actionSetExpertFlag.setEnabled(False)
        self.ui.actionTemplates.setEnabled(False)
        self.ui.treeView.setItemsExpandable(True)
        self.ui.actionSaveConfig.setEnabled(False)
        self.ui.stackedWidget.setCurrentIndex(0)

    @loggingInfo
    def about(self):
        MessageDialogs.showInformationMessage(
            self, "IcepapCMS Version", "IcepapCMS version " + __version__)

    @loggingInfo
    def addLocation(self):
        dlg = DialogAddLocation(self)
        dlg.exec_()
        if dlg.result():
            location = dlg.getData()
            if location == '':
                MessageDialogs.showErrorMessage(
                    self, "Add location", "Location must have a name")
                return
            if self._manager.addLocation(location):
                self.ui.cbLocation.addItem(location)
                if self.ui.cbLocation.count() == 1:
                    self.ui.btnTreeAdd.setEnabled(True)
                    self.ui.btnTreeRemove.setEnabled(True)
                self.ui.cbLocation.setCurrentIndex(
                    self.ui.cbLocation.findText(
                        location, QtCore.Qt.MatchFixedString))
                self.locationChanged(location)
            else:
                MessageDialogs.showErrorMessage(
                    self, "Add location", "Error adding location")

    @loggingInfo
    def deleteLocation(self):
        location = self.ui.cbLocation.currentText()
        delete = MessageDialogs.showYesNoMessage(
            self,
            "Delete location",
            "Remove " +
            location +
            " and all the Icepaps inside?")
        if delete:
            self._manager.deleteLocation(location)
            self.ui.cbLocation.removeItem(self.ui.cbLocation.currentIndex())
            self.buildLocationCombo()

    @loggingInfo
    def buildLocationCombo(self):
        self.ui.cbLocation.clear()
        self.ui.treeView.setModel(None)

        keys = sorted(self._manager.locationList.keys())
        for location_name in keys:
            self.ui.cbLocation.addItem(location_name)
        first_location = self.ui.cbLocation.itemText(0)
        activate = False
        if self.ui.cbLocation.count() > 0:
            activate = True

        self.ui.btnTreeAdd.setEnabled(activate)
        self.ui.btnTreeRemove.setEnabled(activate)
        self.ui.actionAddIcepap.setEnabled(activate)
        self.ui.actionDeleteIcepap.setEnabled(activate)
        if first_location != "":
            self.locationChanged(first_location)

    @loggingInfo
    def locationChanged(self, index):
        location = self.ui.cbLocation.currentText()
        self._manager.changeLocation(location)
        self.buildInitialTree()
        for icepap_system in list(self._manager.IcepapSystemList.values()):
            self.stopIcepap(icepap_system)

    @loggingInfo
    def buildInitialTree(self):
        self._tree_model = IcepapTreeModel(
            self._manager.IcepapSystemList, True)
        self.ui.treeView.setModel(self._tree_model)
        self.context_menu_item = None

    @loggingInfo
    def __contextMenu(self, point):
        modelindex = self.ui.treeView.indexAt(point)
        item = self._tree_model.item(modelindex)
        if not item:
            return

        actions = [
            """self.menu.addAction("Sign driver configuration",
            self.actionSaveConfigMth)""",
            """self.menu.addAction("Solve driver configuration conflict",
            self.contextSolveConflict)""",
            """self.menu.addAction("Delete driver not present",
            self.contextDeleteDriverError)""",
            """self.menu.addAction("New driver. Keep or set to default",
            self.contextSolveNewDriver)""",
            """self.menu.addSeparator()""",
            """self.menu.addAction("Start Icepap system configuration",
            self.contextIcepapStart)""",
            """self.menu.addAction("Rescan Icepap system",
            self.contextIcepapStart)""",
            """self.menu.addAction("Finish Icepap system configuration",
            self.contextIcepapStop)""",
            """self.menu.addSeparator()""",
            """self.menu.addAction("Edit Icepap system information",
            self.contextEditIcepap)""",
            """self.menu.addAction("Delete Icepap system configuration",
            self.btnTreeRemove_on_click)""",
        ]
        self.menu = Qt.QMenu(self)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.menu.setFont(font)
        self.context_menu_item = item
        shown_actions = []
        if item.role == IcepapTreeModel.SYSTEM_OFFLINE:
            shown_actions = [5, 8, 9, 10]
        if item.role == IcepapTreeModel.DRIVER:
            shown_actions = [6, 7, 8, 9, 10]
        elif item.role == IcepapTreeModel.DRIVER_CFG:
            shown_actions = [0, 4, 6, 7, 8, 9, 10]
        elif item.role == IcepapTreeModel.DRIVER_ERROR:
            shown_actions = [2, 4, 6, 7, 8, 9, 10]
        elif item.role == IcepapTreeModel.DRIVER_WARNING:
            shown_actions = [1, 4, 6, 7, 8, 9, 10]
        elif item.role == IcepapTreeModel.DRIVER_NEW:
            shown_actions = [3, 4, 6, 7, 8, 9, 10]
        elif item.role == IcepapTreeModel.SYSTEM or \
                item.role == IcepapTreeModel.CRATE or \
                item.role == IcepapTreeModel.SYSTEM_ERROR or \
                item.role == IcepapTreeModel.SYSTEM_WARNING:
            shown_actions = [6, 7, 8, 9, 10]
        for i in shown_actions:
            exec(actions[i])
        self.menu.popup(self.cursor().pos())

    @loggingInfo
    def contextIcepapStart(self):
        if self.context_menu_item:
            item = self.context_menu_item
            self.scanIcepap(item.getIcepapSystem())
        self.context_menu_item = None

    @loggingInfo
    def contextIcepapStop(self):
        if self.context_menu_item:
            item = self.context_menu_item
            self.stopIcepap(item.getIcepapSystem())
        self.context_menu_item = None

    @loggingInfo
    def contextEditIcepap(self):
        if self.context_menu_item:
            item = self.context_menu_item
            self.editIcepap(item)
        self.context_menu_item = None

    @loggingInfo
    def contextSolveConflict(self):
        if self.context_menu_item:
            item = self.context_menu_item
            self.solveConflict(item)
        self.context_menu_item = None

    @loggingInfo
    def contextDeleteDriverError(self):
        if self.context_menu_item:
            item = self.context_menu_item
            self.deleteDriverError(item)
        self.context_menu_item = None

    @loggingInfo
    def contextSolveNewDriver(self):
        if self.context_menu_item:
            item = self.context_menu_item
            self.solveNewDriver(item)
        self.context_menu_item = None

    @loggingInfo
    def btnTreeAdd_on_click(self):
        location = self.ui.cbLocation.currentText()
        dlg = DialogAddIcepap(self, location)
        dlg.exec_()
        if dlg.result():
            data = dlg.getData()
            self.ui.cbLocation.setCurrentIndex(
                self.ui.cbLocation.findText(
                    data[3], QtCore.Qt.MatchFixedString))
            icepap_system = self._manager.addIcepapSystem(
                data[0], data[1], data[2])
            if icepap_system is not None:
                self._tree_model.addIcepapSystem(
                    icepap_system.name, icepap_system, False)
                self._manager.checkFirmwareVersions(icepap_system)
                self.expandAll(icepap_system.name)

    @loggingInfo
    def actionCopyMth(self):
        self.ui.pageiPapDriver.doCopy()

    @loggingInfo
    def actionPasteMth(self):
        self.ui.pageiPapDriver.doPaste()

    @loggingInfo
    def editIcepap(self, item):
        location = self.ui.cbLocation.currentText()
        dlg = DialogAddIcepap(self, location)
        icepap_system = item.getIcepapSystem()
        dlg.setData(icepap_system.name, icepap_system.host, icepap_system.port,
                    icepap_system.description, location)
        dlg.exec_()
        if dlg.result():
            data = dlg.getData()
            icepap_system.description = str(data[2])
            icepap_system.location_name = str(data[3])

            #item.changeLabel([data[0], data[2]])
            self.ui.stackedWidget.setCurrentIndex(0)
            self.ui.cbLocation.setCurrentIndex(
                self.ui.cbLocation.findText(
                    data[3], QtCore.Qt.MatchFixedString))

    @loggingInfo
    def checkIcepapConnection(self):
        """ this function checks the icepap connection, notifying the user for
        losing or getting connection """
        icepap_systems_changed = self._manager.checkIcepapSystems()
        for icepap_system in icepap_systems_changed:
            # if icepap_system.conflict != Conflict.NO_CONFLICT:
            self.scanIcepap(icepap_system)

    @loggingInfo
    def stopIcepap(self, icepap_system):
        try:
            self.ui.pageiPapDriver.stopTesting()
            if self.refreshTimer is not None:
                self.refreshTimer.stop()
                self.ui.stackedWidget.setCurrentIndex(0)
                self._manager.stopIcepap(icepap_system)
                self._tree_model.updateIcepapSystem(icepap_system, True)
                self.treeSelectByLocation(icepap_system.name)
        except BaseException:
            pass

    @loggingInfo
    def scanIcepap(self, icepap_system):
        """This function scans and Icepap. This means comparing
        the database configurations and the ones in the hardware """
        self.setStatusMessage("Scanning ...")

        # STORM ACCESS TO ALL DRIVERS FROM THE SYSTEM
        icepap_system.loadDriversfromDB()

        conflicts_list = []
        solved_drivers = ""
        conflicts_list.extend(self._manager.scanIcepap(icepap_system))
        if len(conflicts_list) > 0:
            self.setStatusMessage("Configuration conflicts found.")
            for conflict in conflicts_list:
                icepap_system = conflict[1]
                if conflict[0] == Conflict.NO_CONNECTION:
                    icepap_system.setConflict(conflict[0])
                    self.setStatusMessage(
                        icepap_system.name + ": Connection Error")

                elif conflict[0] in [Conflict.DRIVER_AUTOSOLVE,
                                     Conflict.DRIVER_AUTOSOLVE_EXPERT]:
                    driver = icepap_system.getDriver(conflict[2])

                    if conflict[0] == Conflict.DRIVER_AUTOSOLVE:
                        params_modified, params_new_in_driver, \
                            params_old_in_db = \
                            self.get_driver_param_conflicts(
                                driver.icepapsystem_name, driver.addr)
                        self._manager.saveValuesInIcepap(
                            driver, driver.current_cfg.toList(),
                            ignore_values=params_old_in_db)
                        solved_drivers = solved_drivers + \
                            "%s:%d DB->DSP\n" % (driver.icepapsystem_name,
                                                 driver.addr)

                    elif conflict[0] == Conflict.DRIVER_AUTOSOLVE_EXPERT:
                        driver_values = self.getDriverValues(
                            driver.icepapsystem_name, driver.addr)
                        driver.addConfiguration(driver_values)
                        db = StormManager()
                        db.store(driver_values)
                        solved_drivers = solved_drivers + \
                            "%s:%d DB<-DSP\n" % (driver.icepapsystem_name,
                                                 driver.addr)
                        # @TODO NOTE: I KNOW IT IS STILL PENDING TO UPDATE THE
                        #  LABEL AUTOMATICALLY
                        # BUT YOU WILL GET THE NEW NAME BY JUST CLICKING ON
                        # THE TREE ITEM

                    driver.signDriver()
                    driver.setConflict(Conflict.NO_CONFLICT)
                    icepap_system.child_conflicts -= 1

                else:
                    if not conflict[2] is None:
                        self.setStatusMessage("Configuration conflicts found.")
                        driver = icepap_system.getDriver(conflict[2])
                        driver.setConflict(conflict[0])
        else:
            self.setStatusMessage("Scanning complete!. No conflicts found")

        self._manager.checkFirmwareVersions(icepap_system)
        #######################################################################
        # THE LOOP MAY BE DONE ONLY ONCE
        self._tree_model.updateIcepapSystem(icepap_system)
        # SOMETHING SHOULD BE DONE HERE
        self.expandAll(icepap_system.name)
        self.treeSelectByLocation(icepap_system.name)
        if solved_drivers != "":
            MessageDialogs.showInformationMessage(
                self,
                "Solved conflicts",
                "Auto-solved conflicts in drivers:\n" +
                solved_drivers)

    @loggingInfo
    def getDriverDBValues(self, icepap_system, driver_addr):
        dbIcepapSystem = StormManager().getIcepapSystem(icepap_system)
        return dbIcepapSystem.getDriver(
            driver_addr, in_memory=False).startup_cfg

    @loggingInfo
    def getDriverValues(self, icepap_system, driver_addr):
        return self._manager.getDriverConfiguration(icepap_system, driver_addr)

    @loggingInfo
    def get_driver_param_conflicts(self, icepap_system, driver_addr):
        # Get the parameters that have raised the conflict
        driver_values = self.getDriverValues(icepap_system, driver_addr)
        driver_db_values = self.getDriverDBValues(icepap_system, driver_addr)

        params_modified = []
        params_new_in_driver = []
        params_old_in_db = []

        for db_param, db_value in driver_db_values.toList():
            driver_value = driver_values.getParameter(db_param, True)
            if db_value == driver_value:
                pass
            elif driver_value is None:
                params_old_in_db.append((str(db_param), str(db_value)))
            else:
                params_modified.append(
                    (str(db_param), str(db_value), str(driver_value)))

        for driver_param, driver_value in driver_values.toList():
            db_value = driver_db_values.getParameter(driver_param, True)
            if db_value is None:
                params_new_in_driver.append(
                    (str(driver_param), str(driver_value)))

        return params_modified, params_new_in_driver, params_old_in_db

    @loggingInfo
    def solveConflict(self, item):
        driver = item.itemData
        system = driver.icepapsystem_name
        addr = driver.addr
        expert = self._manager._ctrl_icepap.isExpertFlagSet(system, addr)
        expertFlag = (expert == 'YES')
        message = "%s.%d: Set DataBase values?" % (system, addr)
        if expertFlag:
            message = "%s.%d: Set Driver Values?\n" % (system, addr)
            message = message + "FOUND CONFIG WITH EXPERT = YES"

        params_modified, params_new_in_driver, params_old_in_db = \
            self.get_driver_param_conflicts(system, addr)

        widget, table = self.ui.pageiPapDriver.createTableWidget(
            ["Parameter\nname", "Value in\ndatabase",
             "Value in\ndriver board"])
        more_info_dialog = QtWidgets.QDialog(self)
        more_info_dialog.resize(420, 300)
        grid_layout = QtWidgets.QGridLayout()
        grid_layout.addWidget(widget)
        more_info_dialog.setModal(True)
        more_info_dialog.setLayout(grid_layout)

        if len(params_modified) > 0:
            for param, db_value, driver_value in params_modified:
                row = table.rowCount()
                table.insertRow(row)

                param_item = QtWidgets.QTableWidgetItem()
                param_item.setText(param)
                table.setItem(row, 0, param_item)

                db_item = QtWidgets.QTableWidgetItem()
                db_item.setText(db_value)
                table.setItem(row, 1, db_item)

                driver_item = QtWidgets.QTableWidgetItem()
                driver_item.setText(driver_value)
                table.setItem(row, 2, driver_item)

        if len(params_new_in_driver) > 0:
            for param, driver_value in params_new_in_driver:
                row = table.rowCount()
                table.insertRow(row)

                param_item = QtWidgets.QTableWidgetItem()
                param_item.setText(param)
                table.setItem(row, 0, param_item)

                db_item = QtWidgets.QTableWidgetItem()
                db_item.setText("---")
                table.setItem(row, 1, db_item)

                driver_item = QtWidgets.QTableWidgetItem()
                driver_item.setText(driver_value)
                table.setItem(row, 2, driver_item)

        if len(params_old_in_db) > 0:
            for param, db_value in params_old_in_db:
                row = table.rowCount()
                table.insertRow(row)

                param_item = QtWidgets.QTableWidgetItem()
                param_item.setText(param)
                table.setItem(row, 0, param_item)

                db_item = QtWidgets.QTableWidgetItem()
                db_item.setText(db_value)
                table.setItem(row, 1, db_item)

                driver_item = QtWidgets.QTableWidgetItem()
                driver_item.setText("---")
                table.setItem(row, 2, driver_item)

        if not expertFlag:
            dialog = DialogConflictNonExpert(self, more_info_dialog)
        else:
            dialog = DialogConflictExpert(self, more_info_dialog)

        dialog.exec_()

        yes = dialog.result()

        if yes:
            if expert == 'YES':
                driver_values = self.getDriverValues(system, addr)
                driver.addConfiguration(driver_values)
                db = StormManager()
                db.store(driver_values)
            else:
                self._manager.saveValuesInIcepap(
                    driver, driver.current_cfg.toList(),
                    ignore_values=params_old_in_db)
            driver.signDriver()
            item.solveConflict()

        # BY NOW, UPDATE THE ICEPAP NAME MANUALLY
        current_cfg = driver.current_cfg
        label = str(driver.addr) + " " + \
            current_cfg.getParameter(str("IPAPNAME"), True)
        item.changeLabel([label])

        icepap_system = item.itemData.icepap_system
        for driver in icepap_system.drivers:
            if driver.conflict != Conflict.NO_CONFLICT:
                return
        self.setStatusMessage("")

    @loggingInfo
    def solveNewDriver(self, item):
        driver = item.itemData
        system = driver.icepapsystem_name
        addr = driver.addr

        widget, table = self.ui.pageiPapDriver.createTableWidget(
            ["Parameter\nname", "Value in\ndriver board"])
        more_info_dialog = QtWidgets.QDialog(self)
        more_info_dialog.resize(320, 600)
        grid_layout = QtWidgets.QGridLayout()
        grid_layout.addWidget(widget)
        more_info_dialog.setModal(True)
        more_info_dialog.setLayout(grid_layout)

        driver_values = self.getDriverValues(system, addr)
        driver_values_list = driver_values.toList()
        if len(driver_values_list) > 0:
            for param, driver_value in driver_values_list:
                row = table.rowCount()
                table.insertRow(row)

                param_item = QtWidgets.QTableWidgetItem()
                param_item.setText(param)
                table.setItem(row, 0, param_item)

                driver_item = QtWidgets.QTableWidgetItem()
                driver_item.setText(driver_value)
                table.setItem(row, 1, driver_item)

        expert = self._manager._ctrl_icepap.isExpertFlagSet(system, addr)
        expertFlag = (expert == 'YES')
        dialog = DialogNewDriver(self, more_info_dialog, expertFlag)
        dialog.exec_()

        answer = dialog.result()
        if answer not in ["DEFAULT", "DRIVER"]:
            # Since version 1.23, cancelling new driver keeps the system as
            # before
            icepap_system = item.getIcepapSystem()
            icepap_system.removeDriver(item.itemData.addr)
            item.solveConflict()
            self._tree_model.deleteItem(item)
            return False

        if answer == "DEFAULT":
            self._manager.configDriverToDefaults(item.itemData)

        self._manager.updateDriverConfig(item.itemData)
        item.solveConflict()
        driver = item.itemData
        driver.signDriver()
        return True

    @loggingInfo
    def deleteDriverError(self, item):
        delete = MessageDialogs.showYesNoMessage(
            self, "Driver error",
            "Driver not present.\nRemove driver from DB?")
        if delete:
            icepap_system = item.getIcepapSystem()
            icepap_system.removeDriver(item.itemData.addr)
            item.solveConflict()
            self._tree_model.deleteItem(item)

    @loggingInfo
    def refreshTree(self):
        self.ui.pageiPapDriver.stopTesting()
        if self.refreshTimer is not None:
            self.refreshTimer.stop()
        self._manager.reset(self)
        self.initGUI()
        self.ui.stackedWidget.setCurrentIndex(0)

    @loggingInfo
    def btnTreeRemove_on_click(self):
        selectmodel = self.ui.treeView.selectionModel()
        indexes = selectmodel.selectedIndexes()
        if len(indexes) > 0:
            index = indexes[0]
            item = self._tree_model.item(index)
            icepap_system = item.getIcepapSystem()
            delete = MessageDialogs.showYesNoMessage(
                self, "Remove Icepap System",
                "Remove " + icepap_system.name + "?")
            if delete:
                self._tree_model.deleteIcepapSystem(icepap_system.name)
                self._manager.deleteIcepapSystem(icepap_system.name)
                self.clearLocationBar()
                self.refreshTimer.stop()

    @loggingInfo
    def treeview_on_doubleclick(self, modelindex):
        self.locationsPrevious.extend(self.locationsNext)
        item = self._tree_model.item(modelindex)
        if item.role == IcepapTreeModel.DRIVER_WARNING:
            # TODO Investigate why it is not used
            userContinues = self.solveConflict(item)
        elif item.role == IcepapTreeModel.DRIVER_NEW:
            solved = self.solveNewDriver(item)
            if not solved:
                return
        elif item.role == IcepapTreeModel.DRIVER_ERROR:
            self.deleteDriverError(item)
        elif item.role == IcepapTreeModel.SYSTEM_OFFLINE or \
                item.role == IcepapTreeModel.SYSTEM_ERROR:
            self.scanIcepap(item.itemData)

        if item.role == IcepapTreeModel.DRIVER and \
                item.itemData.conflict == Conflict.NO_CONFLICT:
            self.treeview_on_click(modelindex)

    @loggingInfo
    def treeview_on_click(self, modelindex):
        self.locationsPrevious.extend(self.locationsNext)
        self.locationsNext = []
        self.addToPrevious(self.currentLocation)
        self.treeSelectByIndex(modelindex)

    @loggingInfo
    def treeSelectByLocation(self, location):
        self.currentLocation = location
        modelindex = self._tree_model.indexByLocation(location)
        if modelindex is not None:
            selection = Qt.QItemSelection(modelindex, modelindex)
            selectmodel = self.ui.treeView.selectionModel()
            selectmodel.clear()
            selectmodel.select(selection, Qt.QItemSelectionModel.Select)
            self.treeSelectByIndex(modelindex)

    @loggingInfo
    def treeSelectByIndex(self, modelindex):
        item = self._tree_model.item(modelindex)
        self.currentLocation = item.location
        self.ui.actionExport.setEnabled(False)
        self.ui.actionImport.setEnabled(False)
        self.ui.actionHistoricCfg.setEnabled(False)
        self.ui.actionOscilloscope.setEnabled(False)
        self.ui.actionTemplates.setEnabled(False)
        self.ui.actionSaveConfig.setEnabled(False)
        self.ui.actionSetExpertFlag.setEnabled(False)
        self.ui.actionCopy.setEnabled(False)
        self.ui.actionPaste.setEnabled(False)
        self.ui.pageiPapDriver.stopTesting()
        if self.refreshTimer is not None:
            self.refreshTimer.stop()

        # BEFORE CHANGING THE TREE NODE, WE SHOULD CHECK IF THE LAST DRIVER
        # CAN BE SET BACK TO MODE 'OPER' OR NOT
        if self.ui.pageiPapDriver.checkSaveConfigPending():
            self.ui.actionSaveConfig.setEnabled(True)

        if item.role == IcepapTreeModel.DRIVER or \
                item.role == IcepapTreeModel.DRIVER_CFG:

            # THE FILLDATA METHOD KNOWS IF THE BUTTON HAS TO BE ENABLED OR NOT
            self.ui.actionSaveConfig.setEnabled(False)
            self.ui.pageiPapDriver.fillData(item.itemData)
            self.ui.stackedWidget.setCurrentIndex(3)
            self.ui.actionExport.setEnabled(True)
            self.ui.actionImport.setEnabled(True)
            self.ui.actionHistoricCfg.setEnabled(True)
            self.ui.actionOscilloscope.setEnabled(True)
            self._axis_selected = item.itemData
            self.ui.actionTemplates.setEnabled(True)
            self.ui.actionSetExpertFlag.setEnabled(True)
            # ENABLE THE COPY & PASTE ACTIONS
            self.ui.actionCopy.setEnabled(True)
            self.ui.actionPaste.setEnabled(True)

        elif item.role == IcepapTreeModel.SYSTEM or \
                item.role == IcepapTreeModel.SYSTEM_WARNING:
            self.ui.pageiPapSystem.fillData(item.itemData)
            self.ui.stackedWidget.setCurrentIndex(1)
            try:
                self.refreshTimer.timeout.disconnect(
                    self.ui.pageiPapSystem.refresh)
            except Exception:
                pass
            self.refreshTimer.timeout.connect(self.ui.pageiPapSystem.refresh)
            self.refreshTimer.start(2000)
        elif item.role == IcepapTreeModel.CRATE:
            self.ui.pageiPapCrate.fillData(
                item.getIcepapSystem(), int(
                    item.itemLabel[0].value()))
            self.ui.stackedWidget.setCurrentIndex(2)
            try:
                self.refreshTimer.timeout.disconnect(
                    self.ui.pageiPapCrate.refresh)
            except Exception:
                pass
            self.refreshTimer.timeout.connect(self.ui.pageiPapCrate.refresh)
            self.refreshTimer.start(2000)
        else:
            self.ui.stackedWidget.setCurrentIndex(0)

        self.expandIndex(modelindex)

    @loggingInfo
    def actionGoPreviousMth(self):
        location = self.locationsPrevious.pop()
        self.addToNext(self.currentLocation)
        self.treeSelectByLocation(location)

    @loggingInfo
    def actionGoNextMth(self):
        location = self.locationsNext.pop()
        self.addToPrevious(self.currentLocation)
        self.treeSelectByLocation(location)

    @loggingInfo
    def addToPrevious(self, location):
        if not location == "":
            self.locationsPrevious.append(location)
        self.checkGoPreviousActions()

    @loggingInfo
    def addToNext(self, location):
        if not location == "":
            self.locationsNext.append(location)
        self.checkGoPreviousActions()

    @loggingInfo
    def clearLocationBar(self):
        self.ui.stackedWidget.setCurrentIndex(0)
        self.currentLocation = 0
        self.locationsPrevious = []
        self.locationsNext = []
        self.checkGoPreviousActions()

    @loggingInfo
    def checkGoPreviousActions(self):
        if len(self.locationsPrevious) == 0:
            self.ui.actionGoPrevious.setEnabled(False)
        else:
            self.ui.actionGoPrevious.setEnabled(True)

        if len(self.locationsNext) == 0:
            self.ui.actionGoNext.setEnabled(False)
        else:
            self.ui.actionGoNext.setEnabled(True)

    @loggingInfo
    def actionGoUpMth(self):
        selectmodel = self.ui.treeView.selectionModel()
        indexes = selectmodel.selectedIndexes()
        if len(indexes) > 0:
            index = indexes[0]
            modelindex = self._tree_model.parent(index)
            if modelindex.row() > -1:
                self.addToPrevious(self.currentLocation)
                selection = Qt.QItemSelection(modelindex, modelindex)
                selectmodel = self.ui.treeView.selectionModel()
                selectmodel.clear()
                selectmodel.select(selection, Qt.QItemSelectionModel.Select)
                self.treeSelectByIndex(modelindex)

    @loggingInfo
    def actionRefreshMth(self):
        refresh = MessageDialogs.showYesNoMessage(
            self, "Init CMS", "Get all data from Database and lose changes?")
        if refresh:
            self.refreshTree()

    @loggingInfo
    def closeEvent(self, event):
        for child in self.children():
            if isinstance(child, QtWidgets.QDialog):
                child.done(0)

        self.refreshTimer.stop()
        self.ui.stackedWidget.setCurrentIndex(0)
        # Before closing, if any driver was in config mode, be sure if it can
        # be set back to oper mode or some signature is pending

        if self.ui.pageiPapDriver.checkSaveConfigPending():
            signList = self._manager.getDriversToSign()
            if MessageDialogs.showYesNoMessage(
                    self, "Validate Drivers config",
                    "There are driver configurations pending to be "
                    "validated.\nAll changes may be lost\nValidate "
                    "driver configs?."):
                for driver in signList:
                    driver.signDriver()
            else:
                for driver in signList:
                    self._manager.discardDriverChanges(driver)
                    self._manager.endConfiguringDriver(driver)

        if not self._manager.closeAllConnections():
            if MessageDialogs.showYesNoMessage(
                    self, "Storage",
                    "Error closing storage.\nDiscard changes and close?."):
                event.accept()
            else:
                event.ignore()
        else:
            event.accept()

    @loggingInfo
    def setStatusMessage(self, message):
        self.ui.statusbar.showMessage(message)

    @loggingInfo
    def expandAll(self, location):
        parent = self._tree_model.indexByLocation(location)
        if parent:
            childs = self._tree_model.rowCount(parent)
            for row in range(childs):
                index = self._tree_model.index(row, 0, parent)
                self.expandIndex(index)
                item = self._tree_model.item(index)
                self.expandAll(item.location)

    @loggingInfo
    def expandIndex(self, modelindex):
        index = self._tree_model.parent(modelindex)
        while(index.row() > -1):
            self.ui.treeView.expand(index)
            index = self._tree_model.parent(index)

    @loggingInfo
    def actionImportMth(self):
        if self.ui.stackedWidget.currentIndex() == 3:
            self.ui.pageiPapDriver.doImport()

    @loggingInfo
    def actionExportMth(self):
        if self.ui.stackedWidget.currentIndex() == 3:
            self.ui.pageiPapDriver.doExport()

    @loggingInfo
    def actionConsoleMth(self):
        dlg = IcepapConsole(self)
        dlg.show()

    @loggingInfo
    def actionPreferenceMth(self):
        dlg = DialogPreferences(self)
        dlg.exec_()
        if dlg.StorageChanged:
            self._manager.reset(self)
            self.initGUI()

    @loggingInfo
    def actionFirmwareUpgradeMth(self):
        self.clearLocationBar()
        dlg = DialogIcepapProgram(self)
        dlg.exec_()

    def action_osc_clicked(self):
        print(self._axis_selected.icepapsystem_name, self._axis_selected.addr)
        try:
            get_distribution('icepaposc')
        except DistributionNotFound:

            MessageDialogs.showErrorMessage(self, 'IcepapOSC Error',
                                            'IcepapOSC is not installed')
            return

        subprocess.Popen(['icepaposc', '--axis', str(self._axis_selected.addr),
                          self._axis_selected.icepapsystem_name])


    @loggingInfo
    def actionSaveConfigMth(self):
        QtWidgets.QApplication.instance().setOverrideCursor(
            QtGui.QCursor(QtCore.Qt.WaitCursor))
        try:
            if self.ui.stackedWidget.currentIndex() == 0:
                # sign all drivers
                icepap_list = self._manager.getIcepapList()
                for icepap_name, icepap_system in list(icepap_list.items()):
                    icepap_system.signSystem()
                    self._tree_model.layoutChanged.emit()
            elif self.ui.stackedWidget.currentIndex() == 1:
                # sign all icepap system
                self.ui.pageiPapSystem.icepap_system.signSystem()
                self._tree_model.layoutChanged.emit()
            elif self.ui.stackedWidget.currentIndex() == 2:
                # sign all icepap crate
                self.ui.pageiPapCrate.icepap_system.signCrate(
                    self.ui.pageiPapCrate.cratenr)
                self._tree_model.layoutChanged.emit()
            elif self.ui.stackedWidget.currentIndex() == 3:
                # sign driver
                self.ui.pageiPapDriver.signDriver()
                self.ui.actionSaveConfig.setEnabled(False)

        except Exception as e:
            msg = "Some problems saving driver's configuration: {}".format(e)
            self.log.error(msg)
            MessageDialogs.showInformationMessage(self, "Signature", msg)
        QtWidgets.QApplication.instance().restoreOverrideCursor()

    @loggingInfo
    def actionHistoricCfgMth(self):
        if self.ui.actionHistoricCfg.isChecked():
            self.ui.pageiPapDriver.showHistoricWidget()
        else:
            self.ui.pageiPapDriver.hideHistoricWidget()

    @loggingInfo
    def actionTemplatesMth(self):

        # The master catalog file
        master_catalog_file = resource_filename('icepapcms.templates',
                                                'catalog.xml')

        dlg = TemplatesCatalogWidget(
            master_catalog_file, self.ui.pageiPapDriver, self)
        dlg.show()

    @loggingInfo
    def actionHelpMth(self):
        filename = resource_filename('icepapcms.doc',
                                     'IcepapCMSUserManual.pdf')
        webbrowser.open(filename)

    @loggingInfo
    def actionUser_ManualMth(self):
        file_name = resource_filename('icepapcms.doc',
                                      'IcePAP_UserManual.pdf')
        webbrowser.open(file_name)

    @loggingInfo
    def actionHardware_ManualMth(self):
        file_name = resource_filename('icepapcms.doc',
                                      'IcePAP_HardwareManual.pdf')
        webbrowser.open(file_name)

    @loggingInfo
    def menuView_before_show(self):
        self.ui.actionToolbar.setChecked(not self.ui.toolBar.isHidden())
        self.ui.actionTree_Explorer.setChecked(not self.ui.dockTree.isHidden())

    @loggingInfo
    def actionToolbarMth(self):
        self.ui.actionToolbar.setChecked(not self.ui.toolBar.isHidden())
        if not self.ui.actionToolbar.isChecked():
            self.ui.toolBar.show()
        else:
            self.ui.toolBar.close()

    @loggingInfo
    def actionTreeExplorerMth(self):
        self.ui.actionTree_Explorer.setChecked(not self.ui.dockTree.isHidden())
        if not self.ui.actionTree_Explorer.isChecked():
            self.ui.dockTree.show()
        else:
            self.ui.dockTree.close()

    def snapshot(self):
        selected_index = self.ui.treeView.selectedIndexes()
        if selected_index:
            modelindex = selected_index[0]
            item = self._tree_model.item(modelindex)
            icepap_system = item.getIcepapSystem()
            host = icepap_system.host
            port = icepap_system.port
        else:
            host = ''
            port = 5000
        snapshot = DialogSnapshot(self, host, port)
        snapshot.setModal(True)
        snapshot.show()





import logging
import logging.handlers
import queue
import os
import argparse
from .lib import ConfigManager
from .gui.icepapcms import IcepapApp


def get_parser():
    parser = argparse.ArgumentParser('Icepap Configuration Manager '
                                     'System (CMS)')
    parser.add_argument(
        "-e", "--expert", action="store_true", dest="expert",
        help="Full expert interface. False by default")
    parser.add_argument(
        "-s", "--skip-versioncheck", action="store_true",
        dest="skipversioncheck",
        help="Skip the version mismatch check. False by default")
    parser.add_argument(
        "--all-networks", action="store_true", dest="allnets",
        help="Allow all available icepap systems. False by default")
    # parser.add_argument(
    #     "--ldap", action="store_true", dest="ldap",
    #     help="Force LDAP login to get username. False by default")
    parser.add_argument("--debug-level", dest='debug_level', type=str,
                        help='Logging level used:[DEBUG, INFO, WARNING, '
                             'ERROR, CRITICAL]', default='WARNING')
    parser.add_argument("--debug-module", dest='debug_module', type=str,
                        help='Activate the logging for this module, '
                             'eg: icepapcms.gui.ipapconsole',
                        default='')
    parser.add_argument('-d', '--debug', action='store_true',
                        help='Activate the logging only for the icepap '
                             'socket communication.')

    return parser


def configure_logging():
    config_manager = ConfigManager()

    que = queue.Queue(-1)
    queue_handler = logging.handlers.QueueHandler(que)
    log_format = '%(asctime)s - %(message)s'
    log_console = logging.StreamHandler()
    log_console.setFormatter(logging.Formatter(log_format))

    log_filename = os.path.join(config_manager.log_folder, 'log.txt')
    log_file = logging.handlers.RotatingFileHandler(
        log_filename, maxBytes=10000000, backupCount=5)
    log_file.setFormatter(logging.Formatter(log_format))

    # Check if debug_raw_cmd is active
    if config_manager._options.debug:
        module_filter = 'icepap.tcp'
        config_manager._options.debug_level = 'debug'
    else:
        module_filter = config_manager._options.debug_module
    if module_filter:
        log_file.addFilter(logging.Filter(module_filter))
        log_console.addFilter(logging.Filter(module_filter))
    listener = logging.handlers.QueueListener(que, log_console, log_file)
    debug_levels = {'debug': logging.DEBUG,
                    'info': logging.INFO,
                    'warning': logging.WARNING,
                    'error': logging.ERROR,
                    'critical': logging.CRITICAL}
    debug_level = debug_levels[config_manager._options.debug_level.lower()]
    logging.basicConfig(level=debug_level, handlers=[queue_handler])
    return listener


def main():
    config_manager = ConfigManager()
    args = get_parser().parse_args()

    config_manager._options = args
    listener = configure_logging()

    listener.start()
    app = IcepapApp()
    app.start()
    listener.stop()


if __name__ == "__main__":
    main()




#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms
# (https://github.com/ALBA-Synchrotron/icepapcms)
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtGui, QtCore, QtWidgets, uic
from pkg_resources import resource_filename
from icepap import IcePAPController
import os
import logging
from .messagedialogs import MessageDialogs
from ..lib import ConfigManager, IcepapsManager
from ..helpers import loggingInfo


class IcepapConsole(QtWidgets.QDialog):
    log = logging.getLogger('{}.IcepapConsole'.format(__name__))

    @loggingInfo
    def __init__(self, parent=None):
        QtWidgets.QDialog.__init__(self, parent, QtCore.Qt.Window)
        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'ipapconsole.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui,
                   package='icepapcms.gui')
        self.ui.btnDisconnect.setDisabled(True)
        self.ui.console.setDisabled(True)

        # Connect Signals
        self.ui.btnConnect.clicked.connect(self.btnConnect_on_click)
        self.ui.btnDisconnect.clicked.connect(self.btnDisconnect_on_click)
        self.ui.console.commandReceived.connect(self.sendWriteReadCommand)

        self.prompt = "icepap:>"
        font = QtGui.QFont()
        font.setPointSize(8)
        self.setFont(font)
        self.ui.console.setPrompt(self.prompt)
        self.log_folder = None
        self._config = ConfigManager()
        try:
            ipap_cfg = self._config.config[self._config.icepap]
            self.debug = ipap_cfg["debug_enabled"] == str(True)
            self.log_folder = ipap_cfg["log_folder"]
            if not os.path.exists(self.log_folder):
                os.mkdir(self.log_folder)
        except Exception as e:
            self.log.error("icepapconsole_init(): %s", e)

    @loggingInfo
    def btnConnect_on_click(self):
        try:
            addr = str(self.ui.txtHost.text())
            if addr == '':
                MessageDialogs.showErrorMessage(None, 'Host connection',
                                                'Please, write a host '
                                                'name to connect to.')
                return
            if addr.find(":") >= 0:
                aux = addr.split(':')
                host = aux[0]
                port = aux[1]
            else:
                host = addr
                port = "5000"

            if hasattr(self._config, '_options'):
                ipapcontroller = IcepapsManager()
                if not ipapcontroller.host_in_same_subnet(host):
                    MessageDialogs.showInformationMessage(
                        None, "Host connection",
                        "It is not allowed to connect to {}. "
                        "(Check subnet)".format(host))
                    return
            else:
                # JUST RUNNING AS A STAND-ALONE
                pass
            self.prompt = str(host) + " > "
            log_folder = None
            if self.debug:
                log_folder = self.log_folder
            # TODO configure debug folder and level
            self.ipap = IcePAPController(host, int(port))

            self.ui.btnDisconnect.setDisabled(False)
            self.ui.btnConnect.setDisabled(True)
            self.ui.console.setDisabled(False)
            self.ui.console.setFocus()
            self.ui.console.clear()
            self.writeConsole("Connected to Icepap :  " + addr)
            self.ui.console.setPrompt(self.prompt)
            self.writePrompt()
        except Exception as e:
            MessageDialogs.showErrorMessage(None, "Connection error",
                                            "Error connecting "
                                            "to " + addr + "\n" + str(e))

    @loggingInfo
    def btnDisconnect_on_click(self):
        try:
            self.ipap.disconnect()
        except Exception:
            pass
        self.ui.btnDisconnect.setDisabled(True)
        self.ui.btnConnect.setDisabled(False)
        self.ui.console.clear()
        self.ui.console.setDisabled(True)
        self.ui.txtHost.setFocus()

    @loggingInfo
    def writeConsole(self, txt):
        self.ui.console.write(txt + "\n")

    @loggingInfo
    def writePrompt(self):
        self.ui.console.write(self.prompt)

    @loggingInfo
    def sendWriteReadCommand(self, cmd):
        try:
            cmd = str(cmd)
            # determine if the command has an answer
            if len(cmd) == 0:
                # DO NOTHING...
                return
            cmd = cmd.upper()
            if cmd == "QUIT" or cmd == "CLOSE" or cmd == "EXIT":
                self.btnDisconnect_on_click()
                self.close()
                return
            answer = self.ipap.send_cmd(cmd)
            self.log.debug('cmd: %s, answer: %s', cmd, answer)
            if answer is None:
                return
            else:
                res = '\n'.join(answer)
                self.writeConsole(res)

        except Exception as e:
            self.writeConsole("Some exception issuing command "
                              "'{}'.".format(cmd))
            self.writeConsole("               Error is: '{}'.".format(str(e)))

    @loggingInfo
    def closeEvent(self, event):
        self.btnDisconnect_on_click()
        event.accept()


def main():
    import sys
    app = QtWidgets.QApplication(sys.argv)
    console = IcepapConsole(None)
    console.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

# Led.py
# A led widget for PyQT4
# Author: Josep Joan Ribas Prats
# E-mail: jribas at cells DOT es


import sys
from PyQt5 import QtCore, QtWidgets, Qt
from pkg_resources import resource_filename


__all__ = ['Led']


LED_IMAGES = {}
LED_COLORS = ["ledblueoff", "ledblue", "ledgreenoff", "ledgreen",
              "ledredoff", "ledred", "ledyellowoff", "ledyellow",
              "ledorangeoff", "ledorange"]

# Load led images:
for size in ['s24', 's48']:
    LED_IMAGES[size] = {}
    if size == 's24':
        module_name = 'icepapcms.gui.Led.images24'
    else:
        module_name = 'icepapcms.gui.Led.images48'
    for color in LED_COLORS:
        image_filename = '{}.png'.format(color)
        LED_IMAGES[size][color] = Qt.QImage(resource_filename(module_name,
                                                              image_filename))


class Led(QtWidgets.QWidget):

    BLUE, GREEN, RED, YELLOW, ORANGE = list(range(5))
    ON, OFF = list(range(2))
    S24, S48 = list(range(2))
    colors = LED_COLORS

    def __init__(self, parent=None, ledsize=S24, ledcolor=GREEN):

        QtWidgets.QWidget.__init__(self, parent)
        self.ledsize = ledsize
        if ledsize == Led.S48:
            lsize = 48
        else:
            lsize = 24
        self.status = Led.OFF
        self.setObjectName("Led")
        size = QtCore.QSize(QtCore.QRect(0, 0, lsize, lsize).size())
        self.resize(size.expandedTo(self.minimumSizeHint()))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy(0),
                                           QtWidgets.QSizePolicy.Policy(0))
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizePolicy)
        self.setMinimumSize(QtCore.QSize(lsize, lsize))
        self.setMaximumSize(QtCore.QSize(lsize, lsize))

        self.label = QtWidgets.QLabel(self)
        self.label.setGeometry(QtCore.QRect(0, 0, lsize, lsize))

        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy(0),
                                           QtWidgets.QSizePolicy.Policy(0))
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.label.sizePolicy().hasHeightForWidth())

        self.label.setSizePolicy(sizePolicy)

        self.label.setObjectName("label")
        self.changeColor(ledcolor)

        self.retranslateUi(self)

        QtCore.QMetaObject.connectSlotsByName(self)

    # TODO investigate if this method is needed
    # def tr(self, string):
    #     return QtWidgets.QApplication.translate(
    #         "Led", string, None,)

    def retranslateUi(self, Led):
        Led.setWindowTitle(self.tr("Form"))

    def on(self):
        if self.status == Led.OFF:
            self.status = Led.ON
            self.label.setPixmap(Qt.QPixmap.fromImage(self.onled))

    def off(self):
        if self.status == Led.ON:
            self.status = Led.OFF
            self.label.setPixmap(Qt.QPixmap.fromImage(self.offled))

    def changeColor(self, LedColor):
        ledoffcolor = int(LedColor) * 2
        ledoncolor = (int(LedColor) * 2) + 1
        if self.ledsize == Led.S24:
            size = 's24'
        else:
            size = 's48'
        self.offled = LED_IMAGES[size][Led.colors[ledoffcolor]]
        self.onled = LED_IMAGES[size][Led.colors[ledoncolor]]

        if self.status == Led.OFF:
            self.status = Led.ON
            self.off()
        else:
            self.status = Led.OFF
            self.on()


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    lw = Led(None, Led.S48, Led.ORANGE)
    lw.show()
    lw.on()
    sys.exit(app.exec_())



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from storm.locals import Storm, Int, Unicode, ReferenceSet, Reference, DateTime
import datetime
import logging
from ..helpers import loggingInfo

__all__ = ['IcepapDriverCfg']


class IcepapDriverCfg(Storm):
    __storm_table__ = "icepapdrivercfg"
    id = Int(primary=True)
    icepapsystem_name = Unicode()
    driver_addr = Int()
    name = Unicode()
    description = Unicode()
    signature = Unicode()
    date = DateTime()
    """ references """
    icepap_driver = Reference((icepapsystem_name, driver_addr),
                              ("IcepapDriver.icepapsystem_name",
                               "IcepapDriver.addr"))
    parameters = ReferenceSet(id, "CfgParameter.cfg_id")

    log = logging.getLogger('{}.IcepapDriverCfg'.format(__name__))

    @loggingInfo
    def __init__(self, name, description=None):
        if description is None:
            description = str("")
        self.description = str(description)
        self.name = str(name)
        self.date = datetime.datetime.now()
        self.initialize()

    @loggingInfo
    def __storm_loaded__(self):
        self.initialize()
        for cfgpar in self.parameters:
            self._inmemory_parameters[cfgpar.name] = cfgpar

    @loggingInfo
    def initialize(self):
        self._inmemory_parameters = {}

    @loggingInfo
    def setDriver(self, driver):
        self.icepap_driver = driver

    @loggingInfo
    def resetDriver(self):
        self.icepap_driver = None
        self.icepapsystem_name = None
        self.driver_addr = None

    @loggingInfo
    def setSignature(self, signature):
        self.signature = str(signature)

    @loggingInfo
    def getSignature(self):
        return str(self.signature)

    @loggingInfo
    def setParameter(self, name, value):
        name = str(name)
        value = str(value)
        cfgpar = None
        try:
            cfgpar = self.parameters.find(CfgParameter.name == name).one()
        except Exception:
            pass
        if cfgpar is None:
            cfgpar = CfgParameter(self, name, value)
            self.parameters.add(cfgpar)
        else:
            cfgpar.value = value
        self._inmemory_parameters[str(name)] = cfgpar

    @loggingInfo
    def getParameter(self, name, in_memory=False):
        if in_memory:
            if str(name) in self._inmemory_parameters:
                return self._inmemory_parameters[str(name)].value
            else:
                return None
        else:
            cfgpar = self.parameters.find(CfgParameter.name == name).one()
            if cfgpar is not None:
                return cfgpar.value
            else:
                return None

    @loggingInfo
    def toList(self):
        l = []
        for cfgpar in self._inmemory_parameters.values():
            l.append((cfgpar.name, cfgpar.value))
        return l

    @loggingInfo
    def __str__(self):
        text = "Configuration"
        for par in self.parameters:
            text = text + "\n" + par.name + ":\t" + par.value
        return text

    @loggingInfo
    def __ne__(self, other):
        return not self.__eq__(other)

    @loggingInfo
    def __eq__(self, other):
        self_list = self.toList()

        for name, value in self_list:
            other_value = other.getParameter(name, True)
            if name == 'IPAPNAME' or name == 'VER' or name == 'ID':
                pass
            elif other_value is not None:
                if value != other_value:
                    return False
            else:
                return False

        return True


class CfgParameter(Storm):
    __storm_table__ = "cfgparameter"
    __storm_primary__ = ("name", "cfg_id")
    cfg_id = Int()
    name = Unicode()
    value = Unicode()
    """ references """
    driver_cfg = Reference(cfg_id, "IcepapDriverCfg.id")
    log = logging.getLogger('{}.CfgParameter'.format(__name__))

    @loggingInfo
    def __init__(self, cfg, name, value):
        self.driver_cfg = cfg
        self.name = str(name)
        self.value = str(value)
